@@ -120,7 +120,7 @@ class ExpectedImprovement(AnalyticAcquisitionFunction):
             best_f = torch.tensor(best_f)
         self.register_buffer("best_f", best_f)
 
-    @t_batch_mode_transform(expected_q=1)
+    @t_batch_mode_transform(expected_q=1, assert_output_shape=False)
     def forward(self, X: Tensor) -> Tensor:
         r"""Evaluate Expected Improvement on the candidate set X.
 
@@ -163,6 +163,26 @@ class PosteriorMean(AnalyticAcquisitionFunction):
         >>> pm = PM(test_X)
     """
 
+    def __init__(
+        self,
+        model: Model,
+        objective: Optional[ScalarizedObjective] = None,
+        maximize: bool = True,
+    ) -> None:
+        r"""Single-outcome Posterior Mean.
+
+        Args:
+            model: A fitted single-outcome GP model (must be in batch mode if
+                candidate sets X will be)
+            objective: A ScalarizedObjective (optional).
+            maximize: If True, consider the problem a maximization problem. Note
+                that if `maximize=False`, the posterior mean is negated. As a
+                consequence `optimize_acqf(PosteriorMean(gp, maximize=False))`
+                does actually return -1 * minimum of the posterior mean.
+        """
+        super().__init__(model=model, objective=objective)
+        self.maximize = maximize
+
     @t_batch_mode_transform(expected_q=1)
     def forward(self, X: Tensor) -> Tensor:
         r"""Evaluate the posterior mean on the candidate set X.
@@ -176,7 +196,11 @@ class PosteriorMean(AnalyticAcquisitionFunction):
             points `X`.
         """
         posterior = self._get_posterior(X=X)
-        return posterior.mean.view(X.shape[:-2])
+        mean = posterior.mean.view(X.shape[:-2])
+        if self.maximize:
+            return mean
+        else:
+            return -mean
 
 
 class ProbabilityOfImprovement(AnalyticAcquisitionFunction):
@@ -342,7 +366,7 @@ class ConstrainedExpectedImprovement(AnalyticAcquisitionFunction):
         Args:
             model: A fitted single-outcome model.
             best_f: Either a scalar or a `b`-dim Tensor (batch mode) representing
-                the best function value observed so far (assumed noiseless).
+                the best feasible function value observed so far (assumed noiseless).
             objective_index: The index of the objective.
             constraints: A dictionary of the form `{i: [lower, upper]}`, where
                 `i` is the output index, and `lower` and `upper` are lower and upper
@@ -591,3 +615,43 @@ def _get_noiseless_fantasy_model(
     state_dict = deepcopy(model.state_dict())
     fantasy_model.load_state_dict(state_dict)
     return fantasy_model
+
+
+class ScalarizedPosteriorMean(AnalyticAcquisitionFunction):
+    r"""Scalarized Posterior Mean.
+
+    This acquisition function returns a scalarized (across the q-batch)
+    posterior mean given a vector of weights.
+    """
+
+    def __init__(
+        self,
+        model: Model,
+        weights: Tensor,
+        objective: Optional[ScalarizedObjective] = None,
+    ) -> None:
+        r"""Scalarized Posterior Mean.
+
+        Args:
+            model: A fitted single-outcome model.
+            weights: A tensor of shape `q` for scalarization.
+            objective: A ScalarizedObjective. Required for multi-output models.
+        """
+        super().__init__(model=model, objective=objective)
+        self.register_buffer("weights", weights.unsqueeze(dim=0))
+
+    @t_batch_mode_transform()
+    def forward(self, X: Tensor) -> Tensor:
+        r"""Evaluate the scalarized posterior mean on the candidate set X.
+
+        Args:
+            X: A `(b) x q x d`-dim Tensor of `(b)` t-batches of `d`-dim design
+                points each.
+
+        Returns:
+            A `(b)`-dim Tensor of Posterior Mean values at the given design
+            points `X`.
+        """
+        posterior = self._get_posterior(X=X)
+        weighted_means = posterior.mean.squeeze(dim=-1) * self.weights
+        return weighted_means.sum(dim=-1)
