@@ -16,17 +16,22 @@ from typing import Callable, Dict, List, Optional
 
 import torch
 from botorch import settings
-from botorch.acquisition import monte_carlo  # noqa F401
 from botorch.acquisition import analytic, multi_objective
+from botorch.acquisition import monte_carlo  # noqa F401
 from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.multi_objective import monte_carlo as moo_monte_carlo
-from botorch.acquisition.objective import IdentityMCObjective, MCAcquisitionObjective
+from botorch.acquisition.objective import (
+    IdentityMCObjective,
+    MCAcquisitionObjective,
+)
 from botorch.exceptions.errors import UnsupportedError
 from botorch.exceptions.warnings import SamplingWarning
 from botorch.models.model import Model
 from botorch.sampling.samplers import IIDNormalSampler, MCSampler, SobolQMCNormalSampler
-from botorch.utils.multi_objective.box_decomposition import NondominatedPartitioning
-from botorch.utils.transforms import squeeze_last_dim
+from botorch.utils.multi_objective.box_decompositions.non_dominated import (
+    FastNondominatedPartitioning,
+    NondominatedPartitioning,
+)
 from torch import Tensor
 from torch.quasirandom import SobolEngine
 
@@ -78,8 +83,10 @@ def get_acquisition_function(
     else:
         sampler = IIDNormalSampler(num_samples=mc_samples, seed=seed)
     # instantiate and return the requested acquisition function
+    if acquisition_function_name in ("qEI", "qPI"):
+        obj = objective(model.posterior(X_observed).mean)
+        best_f = obj.max(dim=-1).values
     if acquisition_function_name == "qEI":
-        best_f = objective(model.posterior(X_observed).mean).max().item()
         return monte_carlo.qExpectedImprovement(
             model=model,
             best_f=best_f,
@@ -88,7 +95,6 @@ def get_acquisition_function(
             X_pending=X_pending,
         )
     elif acquisition_function_name == "qPI":
-        best_f = objective(model.posterior(X_observed).mean).max().item()
         return monte_carlo.qProbabilityOfImprovement(
             model=model,
             best_f=best_f,
@@ -105,6 +111,7 @@ def get_acquisition_function(
             objective=objective,
             X_pending=X_pending,
             prune_baseline=kwargs.get("prune_baseline", False),
+            marginalize_dim=kwargs.get("marginalize_dim"),
         )
     elif acquisition_function_name == "qSR":
         return monte_carlo.qSimpleRegret(
@@ -122,18 +129,31 @@ def get_acquisition_function(
         )
     elif acquisition_function_name == "qEHVI":
         # pyre-fixme [16]: `Model` has no attribute `train_targets`
-        if "ref_point" not in kwargs:
+        try:
+            ref_point = kwargs["ref_point"]
+        except KeyError:
             raise ValueError("`ref_point` must be specified in kwargs for qEHVI")
-        if "Y" not in kwargs:
+        try:
+            Y = kwargs["Y"]
+        except KeyError:
             raise ValueError("`Y` must be specified in kwargs for qEHVI")
-        ref_point = kwargs["ref_point"]
-        Y = kwargs.get("Y")
         # get feasible points
         if constraints is not None:
             feas = torch.stack([c(Y) <= 0 for c in constraints], dim=-1).all(dim=-1)
             Y = Y[feas]
         obj = objective(Y)
-        partitioning = NondominatedPartitioning(num_outcomes=obj.shape[-1], Y=obj)
+        alpha = kwargs.get("alpha", 0.0)
+        if alpha > 0:
+            partitioning = NondominatedPartitioning(
+                ref_point=torch.as_tensor(ref_point, dtype=Y.dtype, device=Y.device),
+                Y=obj,
+                alpha=alpha,
+            )
+        else:
+            partitioning = FastNondominatedPartitioning(
+                ref_point=torch.as_tensor(ref_point, dtype=Y.dtype, device=Y.device),
+                Y=obj,
+            )
         return moo_monte_carlo.qExpectedHypervolumeImprovement(
             model=model,
             ref_point=ref_point,
@@ -141,6 +161,22 @@ def get_acquisition_function(
             sampler=sampler,
             objective=objective,
             constraints=constraints,
+            X_pending=X_pending,
+        )
+    elif acquisition_function_name == "qNEHVI":
+        if "ref_point" not in kwargs:
+            raise ValueError("`ref_point` must be specified in kwargs for qNEHVI")
+        return moo_monte_carlo.qNoisyExpectedHypervolumeImprovement(
+            model=model,
+            ref_point=kwargs["ref_point"],
+            X_baseline=X_observed,
+            sampler=sampler,
+            objective=objective,
+            constraints=constraints,
+            prune_baseline=True,
+            alpha=kwargs.get("alpha", 0.0),
+            X_pending=X_pending,
+            marginalize_dim=kwargs.get("marginalize_dim"),
         )
     raise NotImplementedError(
         f"Unknown acquisition function {acquisition_function_name}"
@@ -148,7 +184,9 @@ def get_acquisition_function(
 
 
 def get_infeasible_cost(
-    X: Tensor, model: Model, objective: Callable[[Tensor], Tensor] = squeeze_last_dim
+    X: Tensor,
+    model: Model,
+    objective: Optional[Callable[[Tensor, Optional[Tensor]], Tensor]] = None,
 ) -> float:
     r"""Get infeasible cost for a model and objective.
 
@@ -170,6 +208,11 @@ def get_infeasible_cost(
         >>> objective = lambda Y: Y[..., -1] ** 2
         >>> M = get_infeasible_cost(train_X, model, obj)
     """
+    if objective is None:
+
+        def objective(Y: Tensor, X: Optional[Tensor] = None):
+            return Y.squeeze(-1)
+
     posterior = model.posterior(X)
     lb = objective(posterior.mean - 6 * posterior.variance.clamp_min(0).sqrt()).min()
     M = -(lb.clamp_max(0.0))
@@ -202,6 +245,7 @@ def is_nonnegative(acq_function: AcquisitionFunction) -> bool:
             monte_carlo.qProbabilityOfImprovement,
             multi_objective.analytic.ExpectedHypervolumeImprovement,
             multi_objective.monte_carlo.qExpectedHypervolumeImprovement,
+            multi_objective.monte_carlo.qNoisyExpectedHypervolumeImprovement,
         ),
     )
 
@@ -213,6 +257,7 @@ def prune_inferior_points(
     num_samples: int = 2048,
     max_frac: float = 1.0,
     sampler: Optional[MCSampler] = None,
+    marginalize_dim: Optional[int] = None,
 ) -> Tensor:
     r"""Prune points from an input tensor that are unlikely to be the best point.
 
@@ -234,6 +279,9 @@ def prune_inferior_points(
             returned tensor does not exceed `ceil(max_frac * n)`.
         sampler: If provided, will use this customized sampler instead of
             automatically constructing one with `num_samples`.
+        marginalize_dim: A batch dimension that should be marginalized.
+            For example, this is useful when using a batched fully Bayesian
+            model.
 
     Returns:
         A `n' x d` with subset of points in `X`, where
@@ -254,12 +302,12 @@ def prune_inferior_points(
     with torch.no_grad():
         posterior = model.posterior(X=X)
     if sampler is None:
-        if posterior.event_shape.numel() > SobolEngine.MAXDIM:
+        if posterior.base_sample_shape.numel() > SobolEngine.MAXDIM:
             if settings.debug.on():
                 warnings.warn(
-                    f"Sample dimension q*m={posterior.event_shape.numel()} exceeding "
-                    f"Sobol max dimension ({SobolEngine.MAXDIM}). Using iid samples "
-                    "instead.",
+                    f"Sample dimension q*m={posterior.base_sample_shape.numel()} "
+                    f"exceeding Sobol max dimension ({SobolEngine.MAXDIM}). "
+                    "Using iid samples instead.",
                     SamplingWarning,
                 )
             sampler = IIDNormalSampler(num_samples=num_samples)
@@ -268,12 +316,16 @@ def prune_inferior_points(
     samples = sampler(posterior)
     if objective is None:
         objective = IdentityMCObjective()
-    obj_vals = objective(samples)
+    obj_vals = objective(samples, X=X)
     if obj_vals.ndim > 2:
-        # TODO: support batched inputs (req. dealing with ragged tensors)
-        raise UnsupportedError(
-            "Batched models are currently unsupported by prune_inferior_points"
-        )
+        if obj_vals.ndim == 3 and marginalize_dim is not None:
+            obj_vals = obj_vals.mean(dim=marginalize_dim)
+        else:
+            # TODO: support batched inputs (req. dealing with ragged tensors)
+            raise UnsupportedError(
+                "Models with multiple batch dims are currently unsupported by"
+                " prune_inferior_points."
+            )
     is_best = torch.argmax(obj_vals, dim=-1)
     idcs, counts = torch.unique(is_best, return_counts=True)
 
@@ -370,3 +422,22 @@ def expand_trace_observations(
     # change relevant entries of the scaling tensor
     scale_fac[..., q:, fidelity_dims] = sf
     return scale_fac * X_expanded
+
+
+def project_to_sample_points(X: Tensor, sample_points: Tensor) -> Tensor:
+    r"""Augment `X` with sample points at which to take weighted average.
+
+    Args:
+        X: A `batch_shape x 1 x d`-dim Tensor of with one d`-dim design points
+            for each t-batch.
+        sample_points: `p x d'`-dim Tensor (`d' < d`) of `d'`-dim sample points at
+            which to compute the expectation. The `d'`-dims refer to the trailing
+            columns of X.
+    Returns:
+        A `batch_shape x p x d` Tensor where the q-batch includes the `p` sample points.
+    """
+    batch_shape = X.shape[:-2]
+    p, d_prime = sample_points.shape
+    X_new = X.repeat(*(1 for _ in batch_shape), p, 1)  # batch_shape x p x d
+    X_new[..., -d_prime:] = sample_points
+    return X_new
