@@ -1,6 +1,8 @@
 #!/usr/bin/env python3
-
-# Copyright (c) Facebook, Inc. and its affiliates. All Rights Reserved
+# Copyright (c) Facebook, Inc. and its affiliates.
+#
+# This source code is licensed under the MIT license found in the
+# LICENSE file in the root directory of this source tree.
 
 r"""
 Botorch Warnings.
@@ -25,6 +27,12 @@ class InputDataWarning(BotorchWarning):
     pass
 
 
+class CostAwareWarning(BotorchWarning):
+    r"""Warning raised in the context of cost-aware acquisition strategies."""
+
+    pass
+
+
 class OptimizationWarning(BotorchWarning):
     r"""Optimization-releated warnings."""
 
