@@ -15,11 +15,16 @@ from botorch.models.gp_regression import (
     SingleTaskGP,
 )
 from botorch.models.gp_regression_fidelity import SingleTaskMultiFidelityGP
+from botorch.models.gp_regression_mixed import MixedSingleTaskGP
+from botorch.models.higher_order_gp import HigherOrderGP
 from botorch.models.model_list_gp_regression import ModelListGP
-from botorch.models.multitask import FixedNoiseMultiTaskGP, MultiTaskGP
+from botorch.models.multitask import (
+    FixedNoiseMultiTaskGP,
+    MultiTaskGP,
+    KroneckerMultiTaskGP,
+)
 from botorch.models.pairwise_gp import PairwiseGP, PairwiseLaplaceMarginalLogLikelihood
 
-
 __all__ = [
     "AffineDeterministicModel",
     "AffineFidelityCostModel",
@@ -27,6 +32,9 @@ __all__ = [
     "FixedNoiseMultiTaskGP",
     "GenericDeterministicModel",
     "HeteroskedasticSingleTaskGP",
+    "HigherOrderGP",
+    "KroneckerMultiTaskGP",
+    "MixedSingleTaskGP",
     "ModelListGP",
     "MultiTaskGP",
     "PairwiseGP",
