@@ -17,6 +17,7 @@ from botorch import settings
 from botorch.models.gpytorch import BatchedMultiOutputGPyTorchModel
 from botorch.models.transforms.input import InputTransform
 from botorch.models.transforms.outcome import Log, OutcomeTransform
+from botorch.models.utils import fantasize as fantasize_flag
 from botorch.models.utils import validate_input_scaling
 from botorch.sampling.samplers import MCSampler
 from botorch.utils.containers import TrainingData
@@ -90,15 +91,17 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
             >>> train_Y = torch.sin(train_X).sum(dim=1, keepdim=True)
             >>> model = SingleTaskGP(train_X, train_Y)
         """
-        if input_transform is not None:
-            input_transform.to(train_X)
-        transformed_X = self.transform_inputs(
-            X=train_X, input_transform=input_transform
-        )
+        with torch.no_grad():
+            transformed_X = self.transform_inputs(
+                X=train_X, input_transform=input_transform
+            )
         if outcome_transform is not None:
             train_Y, _ = outcome_transform(train_Y)
         self._validate_tensor_args(X=transformed_X, Y=train_Y)
-        validate_input_scaling(train_X=transformed_X, train_Y=train_Y)
+        ignore_X_dims = getattr(self, "_ignore_X_dims_scaling_check", None)
+        validate_input_scaling(
+            train_X=transformed_X, train_Y=train_Y, ignore_X_dims=ignore_X_dims
+        )
         self._set_dimensions(train_X=train_X, train_Y=train_Y)
         train_X, train_Y, _ = self._transform_tensor_args(X=train_X, Y=train_Y)
         if likelihood is None:
@@ -144,7 +147,8 @@ class SingleTaskGP(BatchedMultiOutputGPyTorchModel, ExactGP):
         self.to(train_X)
 
     def forward(self, x: Tensor) -> MultivariateNormal:
-        x = self.transform_inputs(x)
+        if self.training:
+            x = self.transform_inputs(x)
         mean_x = self.mean_module(x)
         covar_x = self.covar_module(x)
         return MultivariateNormal(mean_x, covar_x)
@@ -204,11 +208,10 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
             >>> train_Yvar = torch.full_like(train_Y, 0.2)
             >>> model = FixedNoiseGP(train_X, train_Y, train_Yvar)
         """
-        if input_transform is not None:
-            input_transform.to(train_X)
-        transformed_X = self.transform_inputs(
-            X=train_X, input_transform=input_transform
-        )
+        with torch.no_grad():
+            transformed_X = self.transform_inputs(
+                X=train_X, input_transform=input_transform
+            )
         if outcome_transform is not None:
             train_Y, train_Yvar = outcome_transform(train_Y, train_Yvar)
         self._validate_tensor_args(X=transformed_X, Y=train_Y, Yvar=train_Yvar)
@@ -285,18 +288,24 @@ class FixedNoiseGP(BatchedMultiOutputGPyTorchModel, ExactGP):
             The constructed fantasy model.
         """
         propagate_grads = kwargs.pop("propagate_grads", False)
-        with settings.propagate_grads(propagate_grads):
-            post_X = self.posterior(X, observation_noise=observation_noise, **kwargs)
-        Y_fantasized = sampler(post_X)  # num_fantasies x batch_shape x n' x m
-        # Use the mean of the previous noise values (TODO: be smarter here).
-        # noise should be batch_shape x q x m when X is batch_shape x q x d, and
-        # Y_fantasized is num_fantasies x batch_shape x q x m.
-        noise_shape = Y_fantasized.shape[1:]
-        noise = self.likelihood.noise.mean().expand(noise_shape)
-        return self.condition_on_observations(X=X, Y=Y_fantasized, noise=noise)
+        with fantasize_flag():
+            with settings.propagate_grads(propagate_grads):
+                post_X = self.posterior(
+                    X, observation_noise=observation_noise, **kwargs
+                )
+            Y_fantasized = sampler(post_X)  # num_fantasies x batch_shape x n' x m
+            # Use the mean of the previous noise values (TODO: be smarter here).
+            # noise should be batch_shape x q x m when X is batch_shape x q x d, and
+            # Y_fantasized is num_fantasies x batch_shape x q x m.
+            noise_shape = Y_fantasized.shape[1:]
+            noise = self.likelihood.noise.mean().expand(noise_shape)
+            return self.condition_on_observations(
+                X=self.transform_inputs(X), Y=Y_fantasized, noise=noise
+            )
 
     def forward(self, x: Tensor) -> MultivariateNormal:
-        x = self.transform_inputs(x)
+        if self.training:
+            x = self.transform_inputs(x)
         mean_x = self.mean_module(x)
         covar_x = self.covar_module(x)
         return MultivariateNormal(mean_x, covar_x)
