@@ -139,8 +139,8 @@ class LinearTruncatedFidelityKernel(Kernel):
             self.register_prior(
                 "power_prior",
                 power_prior,
-                lambda: self.power,
-                lambda v: self._set_power(v),
+                lambda m: m.power,
+                lambda m, v: m._set_power(v),
             )
 
         if self.active_dims is not None:
@@ -181,7 +181,6 @@ class LinearTruncatedFidelityKernel(Kernel):
         self.initialize(raw_power=self.raw_power_constraint.inverse_transform(value))
 
     def forward(self, x1: Tensor, x2: Tensor, diag: bool = False, **params) -> Tensor:
-        r""""""
         if params.get("last_dim_is_batch", False):
             raise NotImplementedError(
                 "last_dim_is_batch not yet supported by LinearTruncatedFidelityKernel"
@@ -240,6 +239,10 @@ class LinearTruncatedFidelityKernel(Kernel):
 
     def __getitem__(self, index) -> LinearTruncatedFidelityKernel:
         new_kernel = deepcopy(self)
-        new_kernel.covar_module_unbiased = self.covar_module_unbiased[index]
-        new_kernel.covar_module_biased = self.covar_module_biased[index]
+        new_kernel.covar_module_unbiased = new_kernel.covar_module_unbiased[index]
+        new_kernel.covar_module_biased = new_kernel.covar_module_biased[index]
+        new_kernel.raw_power = torch.nn.Parameter(new_kernel.raw_power[index])
+        new_kernel.batch_shape = new_kernel.batch_shape[
+            1 if isinstance(index, int) else len(index) :
+        ]
         return new_kernel
