@@ -10,10 +10,13 @@ Abstract base module for all BoTorch models.
 
 from __future__ import annotations
 
+import warnings
 from abc import ABC, abstractmethod
 from typing import Any, Dict, List, Optional
 
+import torch
 from botorch import settings
+from botorch.models.utils import fantasize as fantasize_flag
 from botorch.posteriors import Posterior
 from botorch.sampling.samplers import MCSampler
 from botorch.utils.containers import TrainingData
@@ -22,7 +25,19 @@ from torch.nn import Module
 
 
 class Model(Module, ABC):
-    r"""Abstract base class for BoTorch models."""
+    r"""Abstract base class for BoTorch models.
+
+    Args:
+        _has_transformed_inputs: A boolean denoting whether `train_inputs` are currently
+            stored as transformed or not.
+        _original_train_inputs: A Tensor storing the original train inputs for use in
+            `_revert_to_original_inputs`. Note that this is necessary since
+            transform / untransform cycle introduces numerical errors which lead
+            to upstream errors during training.
+    """
+
+    _has_transformed_inputs: bool = False
+    _original_train_inputs: Optional[Tensor] = None
 
     @abstractmethod
     def posterior(
@@ -34,6 +49,10 @@ class Model(Module, ABC):
     ) -> Posterior:
         r"""Computes the posterior over model outputs at the provided points.
 
+        Note: The input transforms should be applied here using
+            `self.transform_inputs(X)` after the `self.eval()` call and before
+            any `model.forward` or `model.likelihood` calls.
+
         Args:
             X: A `b x q x d`-dim Tensor, where `d` is the dimension of the
                 feature space, `q` is the number of points considered jointly,
@@ -51,6 +70,19 @@ class Model(Module, ABC):
         """
         pass  # pragma: no cover
 
+    @property
+    def batch_shape(self) -> torch.Size:
+        r"""The batch shape of the model.
+
+        This is a batch shape from an I/O perspective, independent of the internal
+        representation of the model (as e.g. in BatchedMultiOutputGPyTorchModel).
+        For a model with `m` outputs, a `test_batch_shape x q x d`-shaped input `X`
+        to the `posterior` method returns a Posterior object over an output of
+        shape `broadcast(test_batch_shape, model.batch_shape) x q x m`.
+        """
+        cls_name = self.__class__.__name__
+        raise NotImplementedError(f"{cls_name} does not define batch_shape property")
+
     @property
     def num_outputs(self) -> int:
         r"""The number of outputs of the model."""
@@ -120,10 +152,13 @@ class Model(Module, ABC):
             The constructed fantasy model.
         """
         propagate_grads = kwargs.pop("propagate_grads", False)
-        with settings.propagate_grads(propagate_grads):
-            post_X = self.posterior(X, observation_noise=observation_noise)
-        Y_fantasized = sampler(post_X)  # num_fantasies x batch_shape x n' x m
-        return self.condition_on_observations(X=X, Y=Y_fantasized, **kwargs)
+        with fantasize_flag():
+            with settings.propagate_grads(propagate_grads):
+                post_X = self.posterior(X, observation_noise=observation_noise)
+            Y_fantasized = sampler(post_X)  # num_fantasies x batch_shape x n' x m
+            return self.condition_on_observations(
+                X=self.transform_inputs(X), Y=Y_fantasized, **kwargs
+            )
 
     @classmethod
     def construct_inputs(
@@ -135,7 +170,9 @@ class Model(Module, ABC):
         )
 
     def transform_inputs(
-        self, X: Tensor, input_transform: Optional[Module] = None
+        self,
+        X: Tensor,
+        input_transform: Optional[Module] = None,
     ) -> Tensor:
         r"""Transform inputs.
 
@@ -147,8 +184,53 @@ class Model(Module, ABC):
             A tensor of transformed inputs
         """
         if input_transform is not None:
+            input_transform.to(X)
             return input_transform(X)
         try:
             return self.input_transform(X)
         except AttributeError:
             return X
+
+    def _set_transformed_inputs(self) -> None:
+        r"""Update training inputs with transformed inputs."""
+        if hasattr(self, "input_transform") and not self._has_transformed_inputs:
+            if hasattr(self, "train_inputs"):
+                self._original_train_inputs = self.train_inputs[0]
+                with torch.no_grad():
+                    X_tf = self.input_transform.preprocess_transform(
+                        self.train_inputs[0]
+                    )
+                self.set_train_data(X_tf, strict=False)
+                self._has_transformed_inputs = True
+            else:
+                warnings.warn(
+                    "Could not update `train_inputs` with transformed inputs"
+                    f"since {self.__class__.__name__} does not have a `train_inputs`"
+                    "attribute. Make sure that the `input_transform` is applied to"
+                    "both the train inputs and test inputs.",
+                    RuntimeWarning,
+                )
+
+    def _revert_to_original_inputs(self) -> None:
+        r"""Revert training inputs back to original."""
+        if hasattr(self, "input_transform") and self._has_transformed_inputs:
+            self.set_train_data(self._original_train_inputs, strict=False)
+            self._has_transformed_inputs = False
+
+    def eval(self) -> Model:
+        r"""Puts the model in `eval` mode and sets the transformed inputs."""
+        self._set_transformed_inputs()
+        return super().eval()
+
+    def train(self, mode: bool = True) -> Model:
+        r"""Puts the model in `train` mode and reverts to the original inputs.
+
+        Args:
+            mode: A boolean denoting whether to put in `train` or `eval` mode.
+                If `False`, model is put in `eval` mode.
+        """
+        if mode:
+            self._revert_to_original_inputs()
+        else:
+            self._set_transformed_inputs()
+        return super().train(mode=mode)
