@@ -4,14 +4,21 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from botorch.models.transforms.input import ChainedInputTransform, Normalize, Round
-from botorch.models.transforms.outcome import ChainedOutcomeTransform, Standardize
+from botorch.models.transforms.input import (
+    ChainedInputTransform,
+    Warp,
+    Normalize,
+    Round,
+)
+from botorch.models.transforms.outcome import ChainedOutcomeTransform, Standardize, Log
 
 
 __all__ = [
     "ChainedInputTransform",
     "ChainedOutcomeTransform",
+    "Log",
     "Normalize",
     "Round",
     "Standardize",
+    "Warp",
 ]
