@@ -4,6 +4,14 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+r"""
+Outcome transformations for automatically transforming and un-transforming
+model outputs. Outcome transformations are typically part of a Model and
+applied (i) within the model constructor to transform the train observations
+to the model space, and (ii) in the `Model.posterior` call to untransform
+the model posterior back to the original space.
+"""
+
 from __future__ import annotations
 
 from abc import ABC, abstractmethod
@@ -44,6 +52,23 @@ class OutcomeTransform(Module, ABC):
         """
         pass  # pragma: no cover
 
+    def subset_output(self, idcs: List[int]) -> OutcomeTransform:
+        r"""Subset the transform along the output dimension.
+
+        This functionality is used to properly treat outcome transformations
+        in the `subset_model` functionality.
+
+        Args:
+            idcs: The output indices to subset the transform to.
+
+        Returns:
+            The current outcome transform, subset to the specified output indices.
+        """
+        raise NotImplementedError(
+            f"{self.__class__.__name__} does not implement the "
+            "`subset_output` method"
+        )
+
     def untransform(
         self, Y: Tensor, Yvar: Optional[Tensor] = None
     ) -> Tuple[Tensor, Optional[Tensor]]:
@@ -112,6 +137,19 @@ class ChainedOutcomeTransform(OutcomeTransform, ModuleDict):
             Y, Yvar = tf.forward(Y, Yvar)
         return Y, Yvar
 
+    def subset_output(self, idcs: List[int]) -> OutcomeTransform:
+        r"""Subset the transform along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the transform to.
+
+        Returns:
+            The current outcome transform, subset to the specified output indices.
+        """
+        return self.__class__(
+            **{name: tf.subset_output(idcs=idcs) for name, tf in self.items()}
+        )
+
     def untransform(
         self, Y: Tensor, Yvar: Optional[Tensor] = None
     ) -> Tuple[Tensor, Optional[Tensor]]:
@@ -221,6 +259,38 @@ class Standardize(OutcomeTransform):
         Yvar_tf = Yvar / self._stdvs_sq if Yvar is not None else None
         return Y_tf, Yvar_tf
 
+    def subset_output(self, idcs: List[int]) -> OutcomeTransform:
+        r"""Subset the transform along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the transform to.
+
+        Returns:
+            The current outcome transform, subset to the specified output indices.
+        """
+        new_m = len(idcs)
+        if new_m > self._m:
+            raise RuntimeError(
+                "Trying to subset a transform have more outputs than "
+                " the original transform."
+            )
+        nlzd_idcs = normalize_indices(idcs, d=self._m)
+        new_outputs = None
+        if self._outputs is not None:
+            new_outputs = [i for i in self._outputs if i in nlzd_idcs]
+        new_tf = self.__class__(
+            m=new_m,
+            outputs=new_outputs,
+            batch_shape=self._batch_shape,
+            min_stdv=self._min_stdv,
+        )
+        new_tf.means = self.means[..., nlzd_idcs]
+        new_tf.stdvs = self.stdvs[..., nlzd_idcs]
+        new_tf._stdvs_sq = self._stdvs_sq[..., nlzd_idcs]
+        if not self.training:
+            new_tf.eval()
+        return new_tf
+
     def untransform(
         self, Y: Tensor, Yvar: Optional[Tensor] = None
     ) -> Tuple[Tensor, Optional[Tensor]]:
@@ -256,11 +326,15 @@ class Standardize(OutcomeTransform):
                 "Standardize does not yet support output selection for "
                 "untransform_posterior"
             )
-        if not self._m == posterior.event_shape[-1]:
+        is_mtgp_posterior = False
+        if isinstance(posterior, GPyTorchPosterior):
+            is_mtgp_posterior = posterior._is_mt
+        if not self._m == posterior.event_shape[-1] and not is_mtgp_posterior:
             raise RuntimeError(
                 "Incompatible output dimensions encountered for transform "
-                f"{self._m} and posterior {posterior.event_shape[-1]}"
+                f"{self._m} and posterior {posterior.event_shape[-1]}."
             )
+
         if not isinstance(posterior, GPyTorchPosterior):
             # fall back to TransformedPosterior
             return TransformedPosterior(
@@ -321,6 +395,28 @@ class Log(OutcomeTransform):
         super().__init__()
         self._outputs = outputs
 
+    def subset_output(self, idcs: List[int]) -> OutcomeTransform:
+        r"""Subset the transform along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the transform to.
+
+        Returns:
+            The current outcome transform, subset to the specified output indices.
+        """
+        new_outputs = None
+        if self._outputs is not None:
+            if min(self._outputs + idcs) < 0:
+                raise NotImplementedError(
+                    f"Negative indexing not supported for {self.__class__.__name__} "
+                    "when subsetting outputs and only transforming some outputs."
+                )
+            new_outputs = [i for i in self._outputs if i in idcs]
+        new_tf = self.__class__(outputs=new_outputs)
+        if not self.training:
+            new_tf.eval()
+        return new_tf
+
     def forward(
         self, Y: Tensor, Yvar: Optional[Tensor] = None
     ) -> Tuple[Tensor, Optional[Tensor]]:
@@ -407,3 +503,130 @@ class Log(OutcomeTransform):
             mean_transform=norm_to_lognorm_mean,
             variance_transform=norm_to_lognorm_variance,
         )
+
+
+class Power(OutcomeTransform):
+    r"""Power-transform outcomes.
+
+    Useful if the targets are modeled using a (multivariate) power transform of
+    a Normal distribution. This means that we can use a standard GP model on the
+    power-transformed outcomes and un-transform the model posterior of that GP.
+    """
+
+    def __init__(self, power: float, outputs: Optional[List[int]] = None) -> None:
+        r"""Power-transform outcomes.
+
+        Args:
+            outputs: Which of the outputs to power-transform. If omitted, all
+                outputs will be standardized.
+        """
+        super().__init__()
+        self._outputs = outputs
+        self.power = power
+
+    def subset_output(self, idcs: List[int]) -> OutcomeTransform:
+        r"""Subset the transform along the output dimension.
+
+        Args:
+            idcs: The output indices to subset the transform to.
+
+        Returns:
+            The current outcome transform, subset to the specified output indices.
+        """
+        new_outputs = None
+        if self._outputs is not None:
+            if min(self._outputs + idcs) < 0:
+                raise NotImplementedError(
+                    f"Negative indexing not supported for {self.__class__.__name__} "
+                    "when subsetting outputs and only transforming some outputs."
+                )
+            new_outputs = [i for i in self._outputs if i in idcs]
+        new_tf = self.__class__(power=self.power, outputs=new_outputs)
+        if not self.training:
+            new_tf.eval()
+        return new_tf
+
+    def forward(
+        self, Y: Tensor, Yvar: Optional[Tensor] = None
+    ) -> Tuple[Tensor, Optional[Tensor]]:
+        r"""Power-transform outcomes.
+
+        Args:
+            Y: A `batch_shape x n x m`-dim tensor of training targets.
+            Yvar: A `batch_shape x n x m`-dim tensor of observation noises
+                associated with the training targets (if applicable).
+
+        Returns:
+            A two-tuple with the transformed outcomes:
+
+            - The transformed outcome observations.
+            - The transformed observation noise (if applicable).
+        """
+        Y_tf = Y.pow(self.power)
+        outputs = normalize_indices(self._outputs, d=Y.size(-1))
+        if outputs is not None:
+            Y_tf = torch.stack(
+                [
+                    Y_tf[..., i] if i in outputs else Y[..., i]
+                    for i in range(Y.size(-1))
+                ],
+                dim=-1,
+            )
+        if Yvar is not None:
+            # TODO: Delta method, possibly issue warning
+            raise NotImplementedError(
+                "Power does not yet support transforming observation noise"
+            )
+        return Y_tf, Yvar
+
+    def untransform(
+        self, Y: Tensor, Yvar: Optional[Tensor] = None
+    ) -> Tuple[Tensor, Optional[Tensor]]:
+        r"""Un-transform power-transformed outcomes
+
+        Args:
+            Y: A `batch_shape x n x m`-dim tensor of power-transfomred targets.
+            Yvar: A `batch_shape x n x m`-dim tensor of power-transformed
+                observation noises associated with the training targets
+                (if applicable).
+
+        Returns:
+            A two-tuple with the un-transformed outcomes:
+
+            - The un-power transformed outcome observations.
+            - The un-power transformed observation noise (if applicable).
+        """
+        Y_utf = Y.pow(1.0 / self.power)
+        outputs = normalize_indices(self._outputs, d=Y.size(-1))
+        if outputs is not None:
+            Y_utf = torch.stack(
+                [
+                    Y_utf[..., i] if i in outputs else Y[..., i]
+                    for i in range(Y.size(-1))
+                ],
+                dim=-1,
+            )
+        if Yvar is not None:
+            # TODO: Delta method, possibly issue warning
+            raise NotImplementedError(
+                "Power does not yet support transforming observation noise"
+            )
+        return Y_utf, Yvar
+
+    def untransform_posterior(self, posterior: Posterior) -> Posterior:
+        r"""Un-transform the power-transformed posterior.
+
+        Args:
+            posterior: A posterior in the power-transformed space.
+
+        Returns:
+            The un-transformed posterior.
+        """
+        if self._outputs is not None:
+            raise NotImplementedError(
+                "Power does not yet support output selection for untransform_posterior"
+            )
+        return TransformedPosterior(
+            posterior=posterior,
+            sample_transform=lambda x: x.pow(1.0 / self.power),
+        )
