@@ -7,7 +7,7 @@
 from __future__ import annotations
 
 import warnings
-from typing import Dict, Optional, Union
+from typing import Dict, List, Optional, Tuple, Union
 
 import torch
 from botorch import settings
@@ -16,11 +16,16 @@ from botorch.acquisition.knowledge_gradient import (
     _get_value_function,
     qKnowledgeGradient,
 )
-from botorch.acquisition.monte_carlo import MCAcquisitionFunction
 from botorch.acquisition.utils import is_nonnegative
 from botorch.exceptions.warnings import BadInitialCandidatesWarning, SamplingWarning
 from botorch.models.model import Model
-from botorch.utils.sampling import batched_multinomial, draw_sobol_samples, manual_seed
+from botorch.optim.utils import fix_features
+from botorch.utils.sampling import (
+    batched_multinomial,
+    draw_sobol_samples,
+    manual_seed,
+    get_polytope_samples,
+)
 from botorch.utils.transforms import standardize
 from torch import Tensor
 from torch.quasirandom import SobolEngine
@@ -32,10 +37,15 @@ def gen_batch_initial_conditions(
     q: int,
     num_restarts: int,
     raw_samples: int,
+    fixed_features: Optional[Dict[int, float]] = None,
     options: Optional[Dict[str, Union[bool, float, int]]] = None,
+    inequality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]] = None,
+    equality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]] = None,
 ) -> Tensor:
     r"""Generate a batch of initial conditions for random-restart optimziation.
 
+    TODO: Support t-batches of initial conditions.
+
     Args:
         acq_function: The acquisition function to be optimized.
         bounds: A `2 x d` tensor of lower and upper bounds for each column of `X`.
@@ -44,6 +54,8 @@ def gen_batch_initial_conditions(
             function optimization.
         raw_samples: The number of raw samples to consider in the initialization
             heuristic.
+        fixed_features: A map `{feature_index: value}` for features that
+            should be fixed to a particular value during generation.
         options: Options for initial condition generation. For valid options see
             `initialize_q_batch` and `initialize_q_batch_nonneg`. If `options`
             contains a `nonnegative=True` entry, then `acq_function` is
@@ -52,6 +64,12 @@ def gen_batch_initial_conditions(
             to specify the batch limit for the initialization. This is useful
             for avoiding memory limits when computing the batch posterior over
             raw samples.
+        inequality constraints: A list of tuples (indices, coefficients, rhs),
+            with each tuple encoding an inequality constraint of the form
+            `\sum_i (X[indices[i]] * coefficients[i]) >= rhs`.
+        equality constraints: A list of tuples (indices, coefficients, rhs),
+            with each tuple encoding an inequality constraint of the form
+            `\sum_i (X[indices[i]] * coefficients[i]) = rhs`.
 
     Returns:
         A `num_restarts x q x d` tensor of initial conditions.
@@ -72,7 +90,7 @@ def gen_batch_initial_conditions(
     factor, max_factor = 1, 5
     init_kwargs = {}
     device = bounds.device
-    bounds = bounds.cpu()
+    bounds_cpu = bounds.cpu()
     if "eta" in options:
         init_kwargs["eta"] = options.get("eta")
     if options.get("nonnegative") or is_nonnegative(acq_function):
@@ -95,14 +113,32 @@ def gen_batch_initial_conditions(
     while factor < max_factor:
         with warnings.catch_warnings(record=True) as ws:
             n = raw_samples * factor
-            if effective_dim <= SobolEngine.MAXDIM:
-                X_rnd = draw_sobol_samples(bounds=bounds, n=n, q=q, seed=seed)
+            if inequality_constraints is None and equality_constraints is None:
+                if effective_dim <= SobolEngine.MAXDIM:
+                    X_rnd = draw_sobol_samples(bounds=bounds_cpu, n=n, q=q, seed=seed)
+                else:
+                    with manual_seed(seed):
+                        # load on cpu
+                        X_rnd_nlzd = torch.rand(
+                            n, q, bounds_cpu.shape[-1], dtype=bounds.dtype
+                        )
+                    X_rnd = bounds_cpu[0] + (bounds_cpu[1] - bounds_cpu[0]) * X_rnd_nlzd
             else:
-                with manual_seed(seed):
-                    # load on cpu
-                    X_rnd_nlzd = torch.rand(n * effective_dim, dtype=bounds.dtype)
-                    X_rnd_nlzd = X_rnd_nlzd.view(n, q, bounds.shape[-1])
-                X_rnd = bounds[0] + (bounds[1] - bounds[0]) * X_rnd_nlzd
+                X_rnd = (
+                    get_polytope_samples(
+                        n=n * q,
+                        bounds=bounds,
+                        inequality_constraints=inequality_constraints,
+                        equality_constraints=equality_constraints,
+                        seed=seed,
+                        n_burnin=options.get("n_burnin", 10000),
+                        thinning=options.get("thinning", 32),
+                    )
+                    .view(n, q, -1)
+                    .cpu()
+                )
+
+            X_rnd = fix_features(X_rnd, fixed_features=fixed_features)
             with torch.no_grad():
                 if batch_limit is None:
                     batch_limit = X_rnd.shape[0]
@@ -139,7 +175,10 @@ def gen_one_shot_kg_initial_conditions(
     q: int,
     num_restarts: int,
     raw_samples: int,
+    fixed_features: Optional[Dict[int, float]] = None,
     options: Optional[Dict[str, Union[bool, float, int]]] = None,
+    inequality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]] = None,
+    equality_constraints: Optional[List[Tuple[Tensor, Tensor, float]]] = None,
 ) -> Optional[Tensor]:
     r"""Generate a batch of smart initializations for qKnowledgeGradient.
 
@@ -165,6 +204,8 @@ def gen_one_shot_kg_initial_conditions(
             function optimization.
         raw_samples: The number of raw samples to consider in the initialization
             heuristic.
+        fixed_features: A map `{feature_index: value}` for features that
+            should be fixed to a particular value during generation.
         options: Options for initial condition generation. These contain all
             settings for the standard heuristic initialization from
             `gen_batch_initial_conditions`. In addition, they contain
@@ -173,6 +214,12 @@ def gen_one_shot_kg_initial_conditions(
             restarts and raw samples for solving the posterior objective
             maximization problem, respectively) and `eta` (temperature parameter
             for sampling heuristic from posterior objective maximizers).
+        inequality constraints: A list of tuples (indices, coefficients, rhs),
+            with each tuple encoding an inequality constraint of the form
+            `\sum_i (X[indices[i]] * coefficients[i]) >= rhs`.
+        equality constraints: A list of tuples (indices, coefficients, rhs),
+            with each tuple encoding an inequality constraint of the form
+            `\sum_i (X[indices[i]] * coefficients[i]) = rhs`.
 
     Returns:
         A `num_restarts x q' x d` tensor that can be used as initial conditions
@@ -202,7 +249,10 @@ def gen_one_shot_kg_initial_conditions(
         q=q_aug,
         num_restarts=num_restarts,
         raw_samples=raw_samples,
+        fixed_features=fixed_features,
         options=options,
+        inequality_constraints=inequality_constraints,
+        equality_constraints=equality_constraints,
     )
 
     # compute maximizer of the value function
@@ -210,6 +260,7 @@ def gen_one_shot_kg_initial_conditions(
         model=acq_function.model,
         objective=acq_function.objective,
         sampler=acq_function.inner_sampler,
+        project=getattr(acq_function, "project", None),
     )
     from botorch.optim.optimize import optimize_acqf
 
@@ -219,7 +270,10 @@ def gen_one_shot_kg_initial_conditions(
         q=1,
         num_restarts=options.get("num_inner_restarts", 20),
         raw_samples=options.get("raw_inner_samples", 1024),
+        fixed_features=fixed_features,
         return_best_only=False,
+        inequality_constraints=inequality_constraints,
+        equality_constraints=equality_constraints,
     )
 
     # sampling from the optimizers
@@ -239,6 +293,7 @@ def gen_value_function_initial_conditions(
     num_restarts: int,
     raw_samples: int,
     current_model: Model,
+    fixed_features: Optional[Dict[int, float]] = None,
     options: Optional[Dict[str, Union[bool, float, int]]] = None,
 ) -> Tensor:
     r"""Generate a batch of smart initializations for optimizing
@@ -268,6 +323,8 @@ def gen_value_function_initial_conditions(
             heuristic.
         current_model: The model of the KG acquisition function that was used to
             generate the fantasy model of the value function.
+        fixed_features: A map `{feature_index: value}` for features that
+            should be fixed to a particular value during generation.
         options: Options for initial condition generation. These contain all
             settings for the standard heuristic initialization from
             `gen_batch_initial_conditions`. In addition, they contain
@@ -279,8 +336,8 @@ def gen_value_function_initial_conditions(
 
     Returns:
         A `num_restarts x batch_shape x q x d` tensor that can be used as initial
-        conditions for `optimize_acqf()`. Here `batch_shape` is the
-        `_input_batch_shape` of value function model.
+        conditions for `optimize_acqf()`. Here `batch_shape` is the batch shape
+        of value function model.
 
     Example:
         >>> fant_X = torch.rand(5, 1, 2)
@@ -304,9 +361,8 @@ def gen_value_function_initial_conditions(
     value_function = _get_value_function(
         model=current_model,
         objective=acq_function.objective,
-        sampler=acq_function.sampler
-        if isinstance(acq_function, MCAcquisitionFunction)
-        else None,
+        sampler=getattr(acq_function, "sampler", None),
+        project=getattr(acq_function, "project", None),
     )
     from botorch.optim.optimize import optimize_acqf
 
@@ -316,6 +372,7 @@ def gen_value_function_initial_conditions(
         q=1,
         num_restarts=options.get("num_inner_restarts", 20),
         raw_samples=options.get("raw_inner_samples", 1024),
+        fixed_features=fixed_features,
         return_best_only=False,
         options={
             k: v
@@ -325,7 +382,7 @@ def gen_value_function_initial_conditions(
         },
     )
 
-    batch_shape = acq_function.model._input_batch_shape
+    batch_shape = acq_function.model.batch_shape
     # sampling from the optimizers
     n_value = int((1 - frac_random) * raw_samples)  # number of non-random ICs
     if n_value > 0:
@@ -339,14 +396,20 @@ def gen_value_function_initial_conditions(
         resampled = fantasy_cands[idx]
     else:
         resampled = torch.empty(
-            0, *batch_shape, 1, bounds.shape[-1], dtype=bounds.dtype
+            0,
+            *batch_shape,
+            1,
+            bounds.shape[-1],
+            dtype=fantasy_cands.dtype,
+            device=fantasy_cands.device,
         )
     # add qMC samples
     randomized = draw_sobol_samples(
         bounds=bounds, n=raw_samples - n_value, q=1, batch_shape=batch_shape, seed=seed
-    )
+    ).to(resampled)
     # full set of raw samples
     X_rnd = torch.cat([resampled, randomized], dim=0)
+    X_rnd = fix_features(X_rnd, fixed_features=fixed_features)
 
     # evaluate the raw samples
     with torch.no_grad():
