@@ -6,6 +6,8 @@
 
 from botorch.posteriors.deterministic import DeterministicPosterior
 from botorch.posteriors.gpytorch import GPyTorchPosterior
+from botorch.posteriors.higher_order import HigherOrderGPPosterior
+from botorch.posteriors.multitask import MultitaskGPPosterior
 from botorch.posteriors.posterior import Posterior
 from botorch.posteriors.transformed import TransformedPosterior
 
@@ -13,6 +15,8 @@ from botorch.posteriors.transformed import TransformedPosterior
 __all__ = [
     "DeterministicPosterior",
     "GPyTorchPosterior",
+    "HigherOrderGPPosterior",
+    "MultitaskGPPosterior",
     "Posterior",
     "TransformedPosterior",
 ]
