@@ -10,12 +10,13 @@ Posterior Module to be used with GPyTorch models.
 
 from __future__ import annotations
 
+from contextlib import ExitStack
 from typing import Optional
 
-import gpytorch
 import torch
 from botorch.exceptions.errors import BotorchTensorDimensionError
 from botorch.posteriors.posterior import Posterior
+from gpytorch import settings as gpt_settings
 from gpytorch.distributions import MultitaskMultivariateNormal, MultivariateNormal
 from gpytorch.lazy import BlockDiagLazyTensor, LazyTensor, SumLazyTensor
 from torch import Tensor
@@ -34,6 +35,14 @@ class GPyTorchPosterior(Posterior):
         self.mvn = mvn
         self._is_mt = isinstance(mvn, MultitaskMultivariateNormal)
 
+    @property
+    def base_sample_shape(self) -> torch.Size:
+        r"""The shape of a base sample used for constructing posterior samples."""
+        shape = self.mvn.batch_shape + self.mvn.base_sample_shape
+        if not self._is_mt:
+            shape += torch.Size([1])
+        return shape
+
     @property
     def device(self) -> torch.device:
         r"""The torch device of the posterior."""
@@ -80,7 +89,9 @@ class GPyTorchPosterior(Posterior):
             # remove output dimension in single output case
             if not self._is_mt:
                 base_samples = base_samples.squeeze(-1)
-        with gpytorch.settings.fast_computations(covar_root_decomposition=False):
+        with ExitStack() as es:
+            if gpt_settings._fast_covar_root_decomposition.is_default():
+                es.enter_context(gpt_settings._fast_covar_root_decomposition(False))
             samples = self.mvn.rsample(
                 sample_shape=sample_shape, base_samples=base_samples
             )
@@ -149,7 +160,7 @@ def scalarize_posterior(
     new_mean = offset + (mean @ weights).view(*batch_shape, q)
 
     if q == 1:
-        new_cov = ((cov @ weights) @ weights).view(*batch_shape, q, q)
+        new_cov = weights.unsqueeze(-2) @ (cov @ weights.unsqueeze(-1))
     else:
         # we need to handle potentially different representations of the multi-task mvn
         if mvn._interleaved:
