@@ -49,7 +49,7 @@ class BaseTestProblemBaseTestCase:
 
     def test_forward(self):
         for dtype in (torch.float, torch.double):
-            for batch_shape in (torch.Size(), torch.Size([2])):
+            for batch_shape in (torch.Size(), torch.Size([2]), torch.Size([2, 3])):
                 for f in self.functions:
                     f.to(device=self.device, dtype=dtype)
                     X = torch.rand(*batch_shape, f.dim, device=self.device, dtype=dtype)
@@ -169,6 +169,11 @@ class MockModel(Model):
         event_shape = self._posterior.event_shape
         return event_shape[-1] if len(event_shape) > 0 else 0
 
+    @property
+    def batch_shape(self) -> torch.Size:
+        event_shape = self._posterior.event_shape
+        return event_shape[:-2]
+
     def state_dict(self) -> None:
         pass
 
@@ -186,20 +191,21 @@ class MockAcquisitionFunction:
         self.X_pending = None
 
     def __call__(self, X):
-        return X[..., 0].max(dim=-1)[0]
+        return X[..., 0].max(dim=-1).values
 
     def set_X_pending(self, X_pending: Optional[Tensor] = None):
         self.X_pending = X_pending
 
 
 def _get_random_data(
-    batch_shape: torch.Size, num_outputs: int, n: int = 10, **tkwargs
+    batch_shape: torch.Size, m: int, d: int = 1, n: int = 10, **tkwargs
 ) -> Tuple[Tensor, Tensor]:
-    r"""Generate random data for testing pursposes.
+    r"""Generate random data for testing purposes.
 
     Args:
         batch_shape: The batch shape of the data.
-        num_outputs: The number of outputs.
+        m: The number of outputs.
+        d: The dimension of the input.
         n: The number of data points.
         tkwargs: `device` and `dtype` tensor constructor kwargs.
 
@@ -207,10 +213,12 @@ def _get_random_data(
         A tuple `(train_X, train_Y)` with randomly generated training data.
     """
     rep_shape = batch_shape + torch.Size([1, 1])
-    train_x = torch.linspace(0, 0.95, n, **tkwargs).unsqueeze(-1)
-    train_x = train_x + 0.05 * torch.rand(n, 1, **tkwargs).repeat(rep_shape)
-    train_y = torch.sin(train_x * (2 * math.pi))
-    train_y = train_y + 0.2 * torch.randn(n, num_outputs, **tkwargs).repeat(rep_shape)
+    train_x = torch.stack(
+        [torch.linspace(0, 0.95, n, **tkwargs) for _ in range(d)], dim=-1
+    )
+    train_x = train_x + 0.05 * torch.rand_like(train_x).repeat(rep_shape)
+    train_y = torch.sin(train_x[..., :1] * (2 * math.pi))
+    train_y = train_y + 0.2 * torch.randn(n, m, **tkwargs).repeat(rep_shape)
     return train_x, train_y
 
 
@@ -232,7 +240,7 @@ def _get_test_posterior(
         interleaved: A boolean indicating the format of the
             MultitaskMultivariateNormal
         lazy: A boolean indicating if the posterior should be lazy
-        indepedent: A boolean indicating whether the outputs are independent
+        independent: A boolean indicating whether the outputs are independent
         tkwargs: `device` and `dtype` tensor constructor kwargs.
 
 
@@ -300,6 +308,9 @@ class ConstrainedMultiObjectiveTestProblemBaseTestCase(
         for dtype in (torch.float, torch.double):
             for f in self.functions:
                 f.to(device=self.device, dtype=dtype)
-                X = unnormalize(torch.rand(1, f.dim), bounds=f.bounds)
+                X = unnormalize(
+                    torch.rand(1, f.dim, device=self.device, dtype=dtype),
+                    bounds=f.bounds,
+                )
                 slack = f.evaluate_slack_true(X)
                 self.assertEqual(slack.shape, torch.Size([1, f.num_constraints]))
