@@ -10,6 +10,7 @@ Some basic data transformation helpers.
 
 from __future__ import annotations
 
+import warnings
 from functools import wraps
 from typing import Any, Callable, List, Optional
 
@@ -30,6 +31,11 @@ def squeeze_last_dim(Y: Tensor) -> Tensor:
         >>> Y = torch.rand(4, 3)
         >>> Y_squeezed = squeeze_last_dim(Y)
     """
+    warnings.warn(
+        "`botorch.utils.transforms.squeeze_last_dim` is deprecated. "
+        "Simply use `.squeeze(-1)1 instead.",
+        DeprecationWarning,
+    )
     return Y.squeeze(-1)
 
 
@@ -124,19 +130,63 @@ def normalize_indices(indices: Optional[List[int]], d: int) -> Optional[List[int
     return normalized_indices
 
 
+def _verify_output_shape(acqf: Any, X: Tensor, output: Tensor) -> bool:
+    r"""
+    Performs the output shape checks for `t_batch_mode_transform`. Output shape checks
+    help in catching the errors due to AcquisitionFunction arguments with erroneous
+    return shapes before these errors propagate further down the line.
+
+    This method checks that the `output` shape matches either the t-batch shape of X
+    or the `batch_shape` of `acqf.model`.
+
+    Args:
+        acqf: The AcquisitionFunction object being evaluated.
+        X: The `... x q x d`-dim input tensor with an explicit t-batch.
+        output: The return value of `acqf.method(X, ...)`.
+
+    Returns:
+        True if `output` has the correct shape, False otherwise.
+    """
+    try:
+        return (
+            output.shape == X.shape[:-2]
+            or (output.shape == torch.Size() and X.shape[:-2] == torch.Size([1]))
+            or output.shape == acqf.model.batch_shape
+            # for a batched model, we may unsqueeze a batch dimension in X
+            # corresponding to the model's batch dim. In that case the
+            # acquisition function output should replace the right-most
+            # batch dim of X with the model's batch shape.
+            or output.shape == X.shape[:-3] + acqf.model.batch_shape
+        )
+    except (AttributeError, NotImplementedError):
+        # acqf does not have model or acqf.model does not define `batch_shape`
+        warnings.warn(
+            "Output shape checks failed! Expected output shape to match t-batch shape"
+            f"of X, but got output with shape {output.shape} for X with shape"
+            f"{X.shape}. Make sure that this is the intended behavior!",
+            RuntimeWarning,
+        )
+        return True
+
+
 def t_batch_mode_transform(
     expected_q: Optional[int] = None,
+    assert_output_shape: bool = True,
 ) -> Callable[[Callable[[Any, Tensor], Any]], Callable[[Any, Tensor], Any]]:
     r"""Factory for decorators taking a t-batched `X` tensor.
 
     This method creates decorators for instance methods to transform an input tensor
     `X` to t-batch mode (i.e. with at least 3 dimensions). This assumes the tensor
     has a q-batch dimension. The decorator also checks the q-batch size if `expected_q`
-    is provided.
+    is provided, and the output shape if `assert_output_shape` is `True`.
 
     Args:
         expected_q: The expected q-batch size of X. If specified, this will raise an
-            AssertitionError if X's q-batch size does not equal expected_q.
+            AssertionError if X's q-batch size does not equal expected_q.
+        assert_output_shape: If `True`, this will raise an AssertionError if the
+            output shape does not match either the t-batch shape of X,
+            or the `acqf.model.batch_shape` for acquisition functions using
+            batched models.
 
     Returns:
         The decorated instance method.
@@ -154,10 +204,10 @@ def t_batch_mode_transform(
 
     def decorator(method: Callable[[Any, Tensor], Any]) -> Callable[[Any, Tensor], Any]:
         @wraps(method)
-        def decorated(cls: Any, X: Tensor, **kwargs: Any) -> Any:
+        def decorated(acqf: Any, X: Tensor, *args: Any, **kwargs: Any) -> Any:
             if X.dim() < 2:
                 raise ValueError(
-                    f"{type(cls).__name__} requires X to have at least 2 dimensions,"
+                    f"{type(acqf).__name__} requires X to have at least 2 dimensions,"
                     f" but received X with only {X.dim()} dimensions."
                 )
             elif expected_q is not None and X.shape[-2] != expected_q:
@@ -165,8 +215,21 @@ def t_batch_mode_transform(
                     f"Expected X to be `batch_shape x q={expected_q} x d`, but"
                     f" got X with shape {X.shape}."
                 )
+            # add t-batch dim
             X = X if X.dim() > 2 else X.unsqueeze(0)
-            return method(cls, X, **kwargs)
+            output = method(acqf, X, *args, **kwargs)
+            if assert_output_shape and not _verify_output_shape(
+                acqf=acqf,
+                X=X,
+                output=output,
+            ):
+                raise AssertionError(
+                    "Expected the output shape to match either the t-batch shape of "
+                    "X, or the `model.batch_shape` in the case of acquisition functions"
+                    f"using batch models; but got output with shape {output.shape}"
+                    f"for X with shape {X.shape}."
+                )
+            return output
 
         return decorated
 
