@@ -13,21 +13,14 @@ import tempfile
 import time
 from pathlib import Path
 from subprocess import CalledProcessError
-from typing import Optional
+from typing import Dict, Optional
 
 import nbformat
 from nbconvert import PythonExporter
 
 
 IGNORE = {
-    # some nbs take quite a while, don't run by default
-    "closed_loop_botorch_only.ipynb",
-    "constrained_multi_objective_bo.ipynb",
-    "meta_learning_with_rgpe.ipynb",
-    "multi_objective_bo.ipynb",
-    "preference_bo.ipynb",
-    # some others may require setting paths to local data
-    "vae_mnist.ipynb",
+    "vae_mnist.ipynb",  # requires setting paths to local data
 }
 
 
@@ -40,22 +33,26 @@ def parse_ipynb(file: Path) -> str:
     return script
 
 
-def run_script(script: str) -> None:
+def run_script(script: str, env: Dict[str, str] = None) -> None:
     # need to keep the file around & close it so subprocess does not run into I/O issues
     with tempfile.NamedTemporaryFile(delete=False) as tf:
         tf_name = tf.name
         with open(tf_name, "w") as tmp_script:
             tmp_script.write(script)
-    run_out = subprocess.run(["ipython", tf_name], capture_output=True, text=True)
+    if env is not None:
+        env = {**os.environ, **env}
+    run_out = subprocess.run(
+        ["ipython", tf_name], capture_output=True, text=True, env=env
+    )
     os.remove(tf_name)
     return run_out
 
 
-def run_tutorial(tutorial: Path) -> Optional[str]:
+def run_tutorial(tutorial: Path, smoke_test: bool = False) -> Optional[str]:
     script = parse_ipynb(tutorial)
     tic = time.time()
     print(f"Running tutorial {tutorial.name}.")
-    run_out = run_script(script)
+    run_out = run_script(script, env={"SMOKE_TEST": "true"})
     try:
         run_out.check_returncode()
     except CalledProcessError:
@@ -72,7 +69,14 @@ def run_tutorial(tutorial: Path) -> Optional[str]:
     print(f"Running tutorial {tutorial.name} took {runtime:.2f} seconds.")
 
 
-def run_tutorials(repo_dir: str, include_ignored: bool = False) -> None:
+def run_tutorials(
+    repo_dir: str,
+    include_ignored: bool = False,
+    smoke_test: bool = False,
+) -> None:
+    print(f"Running tutorials in {'smoke test' if smoke_test else 'standard'} mode.")
+    if not smoke_test:
+        print("This may take a long time...")
     tutorial_dir = Path(repo_dir).joinpath("tutorials")
     num_runs = 0
     num_errors = 0
@@ -83,7 +87,7 @@ def run_tutorials(repo_dir: str, include_ignored: bool = False) -> None:
             print(f"Ignoring tutorial {tutorial.name}.")
             continue
         num_runs += 1
-        error = run_tutorial(tutorial)
+        error = run_tutorial(tutorial, smoke_test=smoke_test)
         if error is not None:
             num_errors += 1
             print(error)
@@ -98,10 +102,17 @@ if __name__ == "__main__":
     parser.add_argument(
         "-p", "--path", metavar="path", required=True, help="botorch repo directory."
     )
+    parser.add_argument(
+        "-s", "--smoke", action="store_true", help="Run in smoke test mode."
+    )
     parser.add_argument(
         "--include-ignored",
         action="store_true",
         help="Run all tutorials (incl. ignored).",
     )
     args = parser.parse_args()
-    run_tutorials(repo_dir=args.path, include_ignored=args.include_ignored)
+    run_tutorials(
+        repo_dir=args.path,
+        include_ignored=args.include_ignored,
+        smoke_test=args.smoke,
+    )
