@@ -4,19 +4,22 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from contextlib import ExitStack
 from unittest import mock
 
 import torch
-from botorch.acquisition.analytic import PosteriorMean
+from botorch.acquisition.analytic import PosteriorMean, ScalarizedPosteriorMean
 from botorch.acquisition.cost_aware import GenericCostAwareUtility
 from botorch.acquisition.knowledge_gradient import (
     _get_value_function,
     _split_fantasy_points,
     qKnowledgeGradient,
     qMultiFidelityKnowledgeGradient,
+    ProjectedAcquisitionFunction,
 )
-from botorch.acquisition.monte_carlo import qSimpleRegret
+from botorch.acquisition.monte_carlo import qExpectedImprovement, qSimpleRegret
 from botorch.acquisition.objective import GenericMCObjective, ScalarizedObjective
+from botorch.acquisition.utils import project_to_sample_points
 from botorch.exceptions.errors import UnsupportedError
 from botorch.models import SingleTaskGP
 from botorch.posteriors.gpytorch import GPyTorchPosterior
@@ -24,7 +27,6 @@ from botorch.sampling.samplers import IIDNormalSampler, SobolQMCNormalSampler
 from botorch.utils.testing import BotorchTestCase, MockModel, MockPosterior
 from gpytorch.distributions import MultitaskMultivariateNormal
 
-
 NO = "botorch.utils.testing.MockModel.num_outputs"
 
 
@@ -55,7 +57,7 @@ class TestQKnowledgeGradient(BotorchTestCase):
             self.assertIsNone(qKG.current_value)
             self.assertEqual(qKG.get_augmented_q_batch_size(q=3), 32 + 3)
             # test custom construction
-            obj = GenericMCObjective(lambda Y: Y.mean(dim=-1))
+            obj = GenericMCObjective(lambda Y, X: Y.mean(dim=-1))
             sampler = IIDNormalSampler(num_samples=16)
             X_pending = torch.zeros(2, 2, device=self.device, dtype=dtype)
             qKG = qKnowledgeGradient(
@@ -172,7 +174,7 @@ class TestQKnowledgeGradient(BotorchTestCase):
             self.assertTrue(torch.allclose(val, mean.mean() - current_value, atol=1e-4))
             self.assertTrue(torch.equal(qKG.extract_candidates(X), X[..., :-n_f, :]))
             # test objective (inner MC sampling)
-            objective = GenericMCObjective(objective=lambda Y: Y.norm(dim=-1))
+            objective = GenericMCObjective(objective=lambda Y, X: Y.norm(dim=-1))
             samples = torch.randn(3, 1, 1, device=self.device, dtype=dtype)
             mfm = MockModel(MockPosterior(samples=samples))
             X = torch.rand(n_f + 1, 1, device=self.device, dtype=dtype)
@@ -249,7 +251,7 @@ class TestQKnowledgeGradient(BotorchTestCase):
         qKG = qKnowledgeGradient(
             model=model,
             num_fantasies=1,
-            objective=GenericMCObjective(objective=lambda Y: Y.norm(dim=-1)),
+            objective=GenericMCObjective(objective=lambda Y, X: Y.norm(dim=-1)),
         )
         X = torch.rand(1, 1, device=self.device, dtype=dtype)
         options = {"num_inner_restarts": 1, "raw_inner_samples": 1}
@@ -263,7 +265,7 @@ class TestQKnowledgeGradient(BotorchTestCase):
 
 
 class TestQMultiFidelityKnowledgeGradient(BotorchTestCase):
-    def test_initialize_q_multi_fidelity_knowledge_gradient(self):
+    def test_initialize_qMFKG(self):
         for dtype in (torch.float, torch.double):
             mean = torch.zeros(1, 1, device=self.device, dtype=dtype)
             mm = MockModel(MockPosterior(mean=mean))
@@ -294,11 +296,13 @@ class TestQMultiFidelityKnowledgeGradient(BotorchTestCase):
             X = torch.rand(2, 3, device=self.device, dtype=dtype)
             self.assertTrue(torch.equal(qMFKG.project(X), X))
             self.assertTrue(torch.equal(qMFKG.expand(X), X))
+            self.assertIsNone(qMFKG.valfunc_cls)
+            self.assertIsNone(qMFKG.valfunc_argfac)
             # make sure cost sampling logic works
             self.assertIsInstance(qMFKG.cost_sampler, SobolQMCNormalSampler)
             self.assertEqual(qMFKG.cost_sampler.sample_shape, torch.Size([32]))
 
-    def test_evaluate_q_multi_fidelity_knowledge_gradient(self):
+    def test_evaluate_qMFKG(self):
         for dtype in (torch.float, torch.double):
             # basic test
             n_f = 4
@@ -376,7 +380,7 @@ class TestQMultiFidelityKnowledgeGradient(BotorchTestCase):
             self.assertTrue(torch.allclose(val, val_exp, atol=1e-4))
             self.assertTrue(torch.equal(qMFKG.extract_candidates(X), X[..., :-n_f, :]))
             # test objective (inner MC sampling)
-            objective = GenericMCObjective(objective=lambda Y: Y.norm(dim=-1))
+            objective = GenericMCObjective(objective=lambda Y, X: Y.norm(dim=-1))
             samples = torch.randn(3, 1, 1, device=self.device, dtype=dtype)
             mfm = MockModel(MockPosterior(samples=samples))
             X = torch.rand(n_f + 1, 1, device=self.device, dtype=dtype)
@@ -398,6 +402,127 @@ class TestQMultiFidelityKnowledgeGradient(BotorchTestCase):
             val_exp = mock_util(X, objective(samples) - current_value).mean(dim=0)
             self.assertTrue(torch.allclose(val, val_exp, atol=1e-4))
             self.assertTrue(torch.equal(qMFKG.extract_candidates(X), X[..., :-n_f, :]))
+            # test valfunc_cls and valfunc_argfac
+            d, p, d_prime = 4, 3, 2
+            samples = torch.ones(3, 1, 1, device=self.device, dtype=dtype)
+            mean = torch.tensor(
+                [[0.25], [0.5], [0.75]], device=self.device, dtype=dtype
+            ).expand(n_f, 1, -1, -1)
+            weights = torch.tensor([0.5, 1.0, 1.0], device=self.device, dtype=dtype)
+            mfm = MockModel(MockPosterior(mean=mean, samples=samples))
+            X = torch.rand(n_f * d + d, d, device=self.device, dtype=dtype)
+            sample_points = torch.rand(p, d_prime, device=self.device, dtype=dtype)
+            with mock.patch.object(MockModel, "fantasize", return_value=mfm) as patch_f:
+                with mock.patch(NO, new_callable=mock.PropertyMock) as mock_num_outputs:
+                    mock_num_outputs.return_value = 1
+                    mm = MockModel(None)
+                    qMFKG = qMultiFidelityKnowledgeGradient(
+                        model=mm,
+                        num_fantasies=n_f,
+                        project=lambda X: project_to_sample_points(X, sample_points),
+                        valfunc_cls=ScalarizedPosteriorMean,
+                        valfunc_argfac=lambda model: {"weights": weights},
+                    )
+                    val = qMFKG(X)
+                    patch_f.assert_called_once()
+                    cargs, ckwargs = patch_f.call_args
+                    self.assertEqual(ckwargs["X"].shape, torch.Size([1, 16, 4]))
+                    val_exp = torch.tensor([1.375], device=self.device, dtype=dtype)
+                    self.assertTrue(torch.allclose(val, val_exp, atol=1e-4))
+
+                    patch_f.reset_mock()
+                    qMFKG = qMultiFidelityKnowledgeGradient(
+                        model=mm,
+                        num_fantasies=n_f,
+                        project=lambda X: project_to_sample_points(X, sample_points),
+                        valfunc_cls=qExpectedImprovement,
+                        valfunc_argfac=lambda model: {"best_f": 0.0},
+                    )
+                    val = qMFKG(X)
+                    patch_f.assert_called_once()
+                    cargs, ckwargs = patch_f.call_args
+                    self.assertEqual(ckwargs["X"].shape, torch.Size([1, 16, 4]))
+                    val_exp = torch.tensor([1.0], device=self.device, dtype=dtype)
+                    self.assertTrue(torch.allclose(val, val_exp, atol=1e-4))
+
+    def test_fixed_evaluation_qMFKG(self):
+        # mock test qMFKG.evaluate() with expand, project & cost aware utility
+        for dtype in (torch.float, torch.double):
+            mean = torch.zeros(1, 1, 1, device=self.device, dtype=dtype)
+            mm = MockModel(MockPosterior(mean=mean))
+            cau = GenericCostAwareUtility(mock_util)
+            n_f = 4
+            mean = torch.rand(n_f, 2, 1, 1, device=self.device, dtype=dtype)
+            variance = torch.rand(n_f, 2, 1, 1, device=self.device, dtype=dtype)
+            mfm = MockModel(MockPosterior(mean=mean, variance=variance))
+            with ExitStack() as es:
+                patch_f = es.enter_context(
+                    mock.patch.object(MockModel, "fantasize", return_value=mfm)
+                )
+                mock_num_outputs = es.enter_context(
+                    mock.patch(NO, new_callable=mock.PropertyMock)
+                )
+                es.enter_context(
+                    mock.patch(
+                        "botorch.optim.optimize.optimize_acqf",
+                        return_value=(
+                            torch.ones(1, 1, 1, device=self.device, dtype=dtype),
+                            torch.ones(1, device=self.device, dtype=dtype),
+                        ),
+                    ),
+                )
+                es.enter_context(
+                    mock.patch(
+                        "botorch.generation.gen.gen_candidates_scipy",
+                        return_value=(
+                            torch.ones(1, 1, 1, device=self.device, dtype=dtype),
+                            torch.ones(1, device=self.device, dtype=dtype),
+                        ),
+                    ),
+                )
+
+                mock_num_outputs.return_value = 1
+                qMFKG = qMultiFidelityKnowledgeGradient(
+                    model=mm,
+                    num_fantasies=n_f,
+                    X_pending=torch.rand(1, 1, 1, device=self.device, dtype=dtype),
+                    current_value=torch.zeros(1, device=self.device, dtype=dtype),
+                    cost_aware_utility=cau,
+                    project=lambda X: torch.zeros_like(X),
+                    expand=lambda X: torch.ones_like(X),
+                )
+                val = qMFKG.evaluate(
+                    X=torch.zeros(1, 1, 1, device=self.device, dtype=dtype),
+                    bounds=torch.tensor(
+                        [[0.0], [1.0]], device=self.device, dtype=dtype
+                    ),
+                    num_restarts=1,
+                    raw_samples=1,
+                )
+                patch_f.asset_called_once()
+                cargs, ckwargs = patch_f.call_args
+                self.assertTrue(
+                    torch.equal(
+                        ckwargs["X"],
+                        torch.ones(1, 2, 1, device=self.device, dtype=dtype),
+                    )
+                )
+                self.assertEqual(
+                    val, cau(None, torch.ones(1, device=self.device, dtype=dtype))
+                )
+                # test with defaults - should see no errors
+                qMFKG = qMultiFidelityKnowledgeGradient(
+                    model=mm,
+                    num_fantasies=n_f,
+                )
+                qMFKG.evaluate(
+                    X=torch.zeros(1, 1, 1, device=self.device, dtype=dtype),
+                    bounds=torch.tensor(
+                        [[0.0], [1.0]], device=self.device, dtype=dtype
+                    ),
+                    num_restarts=1,
+                    raw_samples=1,
+                )
 
 
 class TestKGUtils(BotorchTestCase):
@@ -410,12 +535,35 @@ class TestKGUtils(BotorchTestCase):
             self.assertIsInstance(vf, PosteriorMean)
             self.assertIsNone(vf.objective)
             # test SimpleRegret
-            obj = GenericMCObjective(lambda Y: Y.sum(dim=-1))
+            obj = GenericMCObjective(lambda Y, X: Y.sum(dim=-1))
             sampler = IIDNormalSampler(num_samples=2)
             vf = _get_value_function(model=mm, objective=obj, sampler=sampler)
             self.assertIsInstance(vf, qSimpleRegret)
             self.assertEqual(vf.objective, obj)
             self.assertEqual(vf.sampler, sampler)
+            # test with project
+            mock_project = mock.Mock(
+                return_value=torch.ones(1, 1, 1, device=self.device)
+            )
+            vf = _get_value_function(
+                model=mm,
+                objective=obj,
+                sampler=sampler,
+                project=mock_project,
+            )
+            self.assertIsInstance(vf, ProjectedAcquisitionFunction)
+            self.assertEqual(vf.objective, obj)
+            self.assertEqual(vf.sampler, sampler)
+            self.assertEqual(vf.project, mock_project)
+            test_X = torch.rand(1, 1, 1, device=self.device)
+            with mock.patch.object(
+                vf, "base_value_function", __class__=torch.nn.Module, return_value=None
+            ) as patch_bvf:
+                vf(test_X)
+                mock_project.assert_called_once_with(test_X)
+                patch_bvf.assert_called_once_with(
+                    torch.ones(1, 1, 1, device=self.device)
+                )
 
     def test_split_fantasy_points(self):
         for dtype in (torch.float, torch.double):
