@@ -14,8 +14,8 @@ from botorch.acquisition.max_value_entropy_search import (
     _sample_max_value_Thompson,
     qMaxValueEntropy,
     qMultiFidelityMaxValueEntropy,
+    qLowerBoundMaxValueEntropy,
 )
-from botorch.exceptions.errors import UnsupportedError
 from botorch.posteriors import GPyTorchPosterior
 from botorch.sampling.samplers import SobolQMCNormalSampler
 from botorch.utils.testing import BotorchTestCase, MockModel, MockPosterior
@@ -26,9 +26,10 @@ from torch import Tensor
 class MESMockModel(MockModel):
     r"""Mock object that implements dummy methods and feeds through specified outputs"""
 
-    def __init__(self, num_outputs=1):
+    def __init__(self, num_outputs=1, batch_shape=None):
         super().__init__(None)
         self._num_outputs = num_outputs
+        self._batch_shape = torch.Size() if batch_shape is None else batch_shape
 
     def posterior(self, X: Tensor, observation_noise: bool = False) -> MockPosterior:
         m_shape = X.shape[:-1]
@@ -41,11 +42,26 @@ class MESMockModel(MockModel):
         )
         return GPyTorchPosterior(mvn)
 
+    def forward(self, X: Tensor) -> MultivariateNormal:
+        return self.posterior(X).mvn
+
+    @property
+    def batch_shape(self) -> torch.Size:
+        return self._batch_shape
+
     @property
     def num_outputs(self) -> int:
         return self._num_outputs
 
 
+class NoBatchShapeMESMockModel(MESMockModel):
+    # For some reason it's really hard to mock this property to raise a
+    # NotImplementedError, so let's just make a class for it.
+    @property
+    def batch_shape(self) -> torch.Size:
+        raise NotImplementedError
+
+
 class TestMaxValueEntropySearch(BotorchTestCase):
     def test_q_max_value_entropy(self):
         for dtype in (torch.float, torch.double):
@@ -57,18 +73,32 @@ class TestMaxValueEntropySearch(BotorchTestCase):
             candidate_set = torch.rand(1000, 2, device=self.device, dtype=dtype)
 
             # test error in case of batch GP model
+            mm = MESMockModel(batch_shape=torch.Size([2]))
+            with self.assertRaises(NotImplementedError):
+                qMaxValueEntropy(mm, candidate_set, num_mv_samples=10)
+            mm = MESMockModel()
             train_inputs = torch.rand(5, 10, 2, device=self.device, dtype=dtype)
-            mm.train_inputs = (train_inputs,)
+            with self.assertRaises(NotImplementedError):
+                qMaxValueEntropy(
+                    mm, candidate_set, num_mv_samples=10, train_inputs=train_inputs
+                )
+
+            # test that init works if batch_shape is not implemented on the model
+            mm = NoBatchShapeMESMockModel()
+            qMaxValueEntropy(
+                mm,
+                candidate_set,
+                num_mv_samples=10,
+            )
+
+            # test error when number of outputs > 1
+            mm = MESMockModel()
+            mm._num_outputs = 2
             with self.assertRaises(NotImplementedError):
                 qMaxValueEntropy(mm, candidate_set, num_mv_samples=10)
 
-            # test error when number of outputs > 1
-            mm._num_outputs = 2
-            with self.assertRaises(UnsupportedError):
-                qMaxValueEntropy(mm, candidate_set, num_mv_samples=10)
-            mm._num_outputs = 1
-
             # test with X_pending is None
+            mm = MESMockModel()
             train_inputs = torch.rand(10, 2, device=self.device, dtype=dtype)
             mm.train_inputs = (train_inputs,)
             qMVE = qMaxValueEntropy(mm, candidate_set, num_mv_samples=10)
@@ -109,6 +139,60 @@ class TestMaxValueEntropySearch(BotorchTestCase):
                 )
                 patch_f.assert_called_once()
 
+    def test_q_lower_bound_max_value_entropy(self):
+        for dtype in (torch.float, torch.double):
+            torch.manual_seed(7)
+            mm = MESMockModel()
+            with self.assertRaises(TypeError):
+                qLowerBoundMaxValueEntropy(mm)
+
+            candidate_set = torch.rand(1000, 2, device=self.device, dtype=dtype)
+
+            # test error in case of batch GP model
+            # train_inputs = torch.rand(5, 10, 2, device=self.device, dtype=dtype)
+            # mm.train_inputs = (train_inputs,)
+            mm = MESMockModel(batch_shape=torch.Size([2]))
+            with self.assertRaises(NotImplementedError):
+                qLowerBoundMaxValueEntropy(mm, candidate_set, num_mv_samples=10)
+
+            # test error when number of outputs > 1
+            mm = MESMockModel()
+            mm._num_outputs = 2
+            with self.assertRaises(NotImplementedError):
+                qLowerBoundMaxValueEntropy(mm, candidate_set, num_mv_samples=10)
+            mm._num_outputs = 1
+
+            # test with X_pending is None
+            mm = MESMockModel()
+            train_inputs = torch.rand(10, 2, device=self.device, dtype=dtype)
+            mm.train_inputs = (train_inputs,)
+            qGIBBON = qLowerBoundMaxValueEntropy(mm, candidate_set, num_mv_samples=10)
+
+            # test initialization
+            self.assertEqual(qGIBBON.num_mv_samples, 10)
+            self.assertEqual(qGIBBON.use_gumbel, True)
+            self.assertEqual(qGIBBON.posterior_max_values.shape, torch.Size([10, 1]))
+
+            # test evaluation
+            X = torch.rand(1, 2, device=self.device, dtype=dtype)
+            self.assertEqual(qGIBBON(X).shape, torch.Size([1]))
+
+            # test with use_gumbel = False
+            qGIBBON = qLowerBoundMaxValueEntropy(
+                mm, candidate_set, num_mv_samples=10, use_gumbel=False
+            )
+            self.assertEqual(qGIBBON(X).shape, torch.Size([1]))
+
+            # test with X_pending is not None
+            qGIBBON = qLowerBoundMaxValueEntropy(
+                mm,
+                candidate_set,
+                num_mv_samples=10,
+                use_gumbel=False,
+                X_pending=torch.rand(1, 2, device=self.device, dtype=dtype),
+            )
+            self.assertEqual(qGIBBON(X).shape, torch.Size([1]))
+
     def test_q_multi_fidelity_max_value_entropy(self):
         for dtype in (torch.float, torch.double):
             torch.manual_seed(7)
