@@ -21,13 +21,17 @@ from botorch.acquisition.utils import (
     expand_trace_observations,
     get_acquisition_function,
     get_infeasible_cost,
+    project_to_sample_points,
     project_to_target_fidelity,
     prune_inferior_points,
 )
 from botorch.exceptions.errors import UnsupportedError
 from botorch.exceptions.warnings import SamplingWarning
 from botorch.sampling.samplers import IIDNormalSampler, SobolQMCNormalSampler
-from botorch.utils.multi_objective.box_decomposition import NondominatedPartitioning
+from botorch.utils.multi_objective.box_decompositions.non_dominated import (
+    FastNondominatedPartitioning,
+    NondominatedPartitioning,
+)
 from botorch.utils.testing import BotorchTestCase, MockModel, MockPosterior
 from torch import Tensor
 
@@ -58,6 +62,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
 
     @mock.patch(f"{monte_carlo.__name__}.qExpectedImprovement")
     def test_GetQEI(self, mock_acqf):
+        self.model = MockModel(MockPosterior(mean=torch.zeros(1, 2)))
         acqf = get_acquisition_function(
             acquisition_function_name="qEI",
             model=self.model,
@@ -66,6 +71,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
             X_pending=self.X_pending,
             mc_samples=self.mc_samples,
             seed=self.seed,
+            marginalize_dim=0,
         )
         self.assertTrue(acqf == mock_acqf.return_value)
         best_f = self.objective(self.model.posterior(self.X_observed).mean).max().item()
@@ -76,6 +82,19 @@ class TestGetAcquisitionFunction(BotorchTestCase):
             objective=self.objective,
             X_pending=self.X_pending,
         )
+        # test batched model
+        self.model = MockModel(MockPosterior(mean=torch.zeros(1, 2, 1)))
+        acqf = get_acquisition_function(
+            acquisition_function_name="qEI",
+            model=self.model,
+            objective=self.objective,
+            X_observed=self.X_observed,
+            X_pending=self.X_pending,
+            mc_samples=self.mc_samples,
+            seed=self.seed,
+        )
+        self.assertTrue(acqf == mock_acqf.return_value)
+        # test batched model without marginalize dim
         args, kwargs = mock_acqf.call_args
         self.assertEqual(args, ())
         sampler = kwargs["sampler"]
@@ -87,6 +106,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
     @mock.patch(f"{monte_carlo.__name__}.qProbabilityOfImprovement")
     def test_GetQPI(self, mock_acqf):
         # basic test
+        self.model = MockModel(MockPosterior(mean=torch.zeros(1, 2)))
         acqf = get_acquisition_function(
             acquisition_function_name="qPI",
             model=self.model,
@@ -145,6 +165,18 @@ class TestGetAcquisitionFunction(BotorchTestCase):
             seed=2,
             tau=1.0,
         )
+        # test batched model
+        self.model = MockModel(MockPosterior(mean=torch.zeros(1, 2, 1)))
+        acqf = get_acquisition_function(
+            acquisition_function_name="qPI",
+            model=self.model,
+            objective=self.objective,
+            X_observed=self.X_observed,
+            X_pending=self.X_pending,
+            mc_samples=self.mc_samples,
+            seed=self.seed,
+        )
+        self.assertTrue(acqf == mock_acqf.return_value)
 
     @mock.patch(f"{monte_carlo.__name__}.qNoisyExpectedImprovement")
     def test_GetQNEI(self, mock_acqf):
@@ -157,6 +189,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
             X_pending=self.X_pending,
             mc_samples=self.mc_samples,
             seed=self.seed,
+            marginalize_dim=0,
         )
         self.assertTrue(acqf == mock_acqf.return_value)
         self.assertTrue(mock_acqf.call_count, 1)
@@ -168,6 +201,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
         self.assertIsInstance(sampler, SobolQMCNormalSampler)
         self.assertEqual(sampler.sample_shape, torch.Size([self.mc_samples]))
         self.assertEqual(sampler.seed, 1)
+        self.assertEqual(kwargs["marginalize_dim"], 0)
         # test with non-qmc, no X_pending
         acqf = get_acquisition_function(
             acquisition_function_name="qNEI",
@@ -341,6 +375,7 @@ class TestGetAcquisitionFunction(BotorchTestCase):
             ref_point=self.ref_point,
             partitioning=mock.ANY,
             sampler=mock.ANY,
+            X_pending=self.X_pending,
         )
         args, kwargs = mock_acqf.call_args
         self.assertEqual(args, ())
@@ -369,10 +404,28 @@ class TestGetAcquisitionFunction(BotorchTestCase):
         self.assertIsInstance(sampler, IIDNormalSampler)
         self.assertIsInstance(kwargs["objective"], DummyMCMultiOutputObjective)
         partitioning = kwargs["partitioning"]
-        self.assertIsInstance(partitioning, NondominatedPartitioning)
-
+        self.assertIsInstance(partitioning, FastNondominatedPartitioning)
         self.assertEqual(sampler.sample_shape, torch.Size([self.mc_samples]))
         self.assertEqual(sampler.seed, 2)
+        # test that approximate partitioning is used when alpha > 0
+        # test with non-qmc
+        acqf = get_acquisition_function(
+            acquisition_function_name="qEHVI",
+            model=self.model,
+            objective=self.mo_objective,
+            X_observed=self.X_observed,
+            X_pending=self.X_pending,
+            mc_samples=self.mc_samples,
+            seed=2,
+            qmc=False,
+            ref_point=self.ref_point,
+            Y=self.Y,
+            alpha=0.1,
+        )
+        _, kwargs = mock_acqf.call_args
+        partitioning = kwargs["partitioning"]
+        self.assertIsInstance(partitioning, NondominatedPartitioning)
+        self.assertEqual(partitioning.alpha, 0.1)
         # test constraints
         acqf = get_acquisition_function(
             acquisition_function_name="qEHVI",
@@ -389,7 +442,90 @@ class TestGetAcquisitionFunction(BotorchTestCase):
         )
         _, kwargs = mock_acqf.call_args
         partitioning = kwargs["partitioning"]
-        self.assertEqual(partitioning._pareto_Y.shape[0], 0)
+        self.assertEqual(partitioning.pareto_Y.shape[0], 0)
+
+    @mock.patch(f"{moo_monte_carlo.__name__}.qNoisyExpectedHypervolumeImprovement")
+    def test_GetQNEHVI(self, mock_acqf):
+        # make sure ref_point is specified
+        with self.assertRaises(ValueError):
+            acqf = get_acquisition_function(
+                acquisition_function_name="qNEHVI",
+                model=self.model,
+                objective=self.objective,
+                X_observed=self.X_observed,
+                X_pending=self.X_pending,
+                mc_samples=self.mc_samples,
+                seed=self.seed,
+            )
+        acqf = get_acquisition_function(
+            acquisition_function_name="qNEHVI",
+            model=self.model,
+            objective=self.objective,
+            X_observed=self.X_observed,
+            X_pending=self.X_pending,
+            mc_samples=self.mc_samples,
+            seed=self.seed,
+            ref_point=self.ref_point,
+        )
+        self.assertTrue(acqf == mock_acqf.return_value)
+        mock_acqf.assert_called_once_with(
+            constraints=None,
+            model=self.model,
+            X_baseline=self.X_observed,
+            objective=self.objective,
+            ref_point=self.ref_point,
+            sampler=mock.ANY,
+            prune_baseline=True,
+            alpha=0.0,
+            X_pending=self.X_pending,
+            marginalize_dim=None,
+        )
+        args, kwargs = mock_acqf.call_args
+        self.assertEqual(args, ())
+        sampler = kwargs["sampler"]
+        self.assertIsInstance(sampler, SobolQMCNormalSampler)
+        self.assertEqual(sampler.sample_shape, torch.Size([self.mc_samples]))
+        self.assertEqual(sampler.seed, 1)
+        # test with non-qmc
+        acqf = get_acquisition_function(
+            acquisition_function_name="qNEHVI",
+            model=self.model,
+            objective=self.objective,
+            X_observed=self.X_observed,
+            X_pending=self.X_pending,
+            mc_samples=self.mc_samples,
+            seed=2,
+            qmc=False,
+            ref_point=self.ref_point,
+        )
+        self.assertTrue(mock_acqf.call_count, 2)
+        args, kwargs = mock_acqf.call_args
+        self.assertEqual(args, ())
+        self.assertEqual(kwargs["ref_point"], self.ref_point)
+        sampler = kwargs["sampler"]
+        self.assertIsInstance(sampler, IIDNormalSampler)
+        ref_point = kwargs["ref_point"]
+        self.assertEqual(ref_point, self.ref_point)
+
+        self.assertEqual(sampler.sample_shape, torch.Size([self.mc_samples]))
+        self.assertEqual(sampler.seed, 2)
+
+        # test passing alpha
+        acqf = get_acquisition_function(
+            acquisition_function_name="qNEHVI",
+            model=self.model,
+            objective=self.objective,
+            X_observed=self.X_observed,
+            X_pending=self.X_pending,
+            mc_samples=self.mc_samples,
+            seed=2,
+            qmc=False,
+            ref_point=self.ref_point,
+            alpha=0.01,
+        )
+        self.assertTrue(mock_acqf.call_count, 3)
+        args, kwargs = mock_acqf.call_args
+        self.assertEqual(kwargs["alpha"], 0.01)
 
     def test_GetUnknownAcquisitionFunction(self):
         with self.assertRaises(NotImplementedError):
@@ -421,6 +557,9 @@ class TestGetInfeasibleCost(BotorchTestCase):
                 X=X, model=mm, objective=lambda Y: Y.squeeze(-1) - 5.0
             )
             self.assertEqual(M, 6.0)
+            # test default objective (squeeze last dim)
+            M2 = get_infeasible_cost(X=X, model=mm)
+            self.assertEqual(M2, 1.0)
 
 
 class TestPruneInferiorPoints(BotorchTestCase):
@@ -435,8 +574,16 @@ class TestPruneInferiorPoints(BotorchTestCase):
             # test that a batched X raises errors
             with self.assertRaises(UnsupportedError):
                 prune_inferior_points(model=mm, X=X.expand(2, 3, 2))
-            # test that a batched model raises errors (event shape is `q x t` = 3 x 1)
+            # test marginalize_dim
             mm2 = MockModel(MockPosterior(samples=samples.expand(2, 3, 1)))
+            X_pruned = prune_inferior_points(model=mm2, X=X, marginalize_dim=-3)
+            with self.assertRaises(UnsupportedError):
+                # test error raised when marginalize_dim is not specified with
+                # a batch model
+                prune_inferior_points(model=mm2, X=X)
+            self.assertTrue(torch.equal(X_pruned, X[[-1]]))
+            # test that a batched model raises errors when there are multiple batch dims
+            mm2 = MockModel(MockPosterior(samples=samples.expand(1, 2, 3, 1)))
             with self.assertRaises(UnsupportedError):
                 prune_inferior_points(model=mm2, X=X)
             # test that invalid max_frac is checked properly
@@ -446,7 +593,7 @@ class TestPruneInferiorPoints(BotorchTestCase):
             X_pruned = prune_inferior_points(model=mm, X=X)
             self.assertTrue(torch.equal(X_pruned, X[[-1]]))
             # test custom objective
-            neg_id_obj = GenericMCObjective(lambda X: -(X.squeeze(-1)))
+            neg_id_obj = GenericMCObjective(lambda Y, X: -(Y.squeeze(-1)))
             X_pruned = prune_inferior_points(model=mm, X=X, objective=neg_id_obj)
             self.assertTrue(torch.equal(X_pruned, X[[0]]))
             # test non-repeated samples (requires mocking out MockPosterior's rsample)
@@ -463,7 +610,7 @@ class TestPruneInferiorPoints(BotorchTestCase):
             with mock.patch.object(MockPosterior, "rsample", return_value=samples):
                 mm = MockModel(MockPosterior(samples=samples))
                 X_pruned = prune_inferior_points(model=mm, X=X, max_frac=2 / 3)
-            if self.device == torch.device("cuda"):
+            if self.device.type == "cuda":
                 # sorting has different order on cuda
                 self.assertTrue(torch.equal(X_pruned, torch.stack([X[2], X[1]], dim=0)))
             else:
@@ -478,11 +625,13 @@ class TestPruneInferiorPoints(BotorchTestCase):
             with ExitStack() as es:
                 mock_event_shape = es.enter_context(
                     mock.patch(
-                        "botorch.utils.testing.MockPosterior.event_shape",
+                        "botorch.utils.testing.MockPosterior.base_sample_shape",
                         new_callable=mock.PropertyMock,
                     )
                 )
-                mock_event_shape.return_value = torch.Size([1, 1, 1112])
+                mock_event_shape.return_value = torch.Size(
+                    [1, 1, torch.quasirandom.SobolEngine.MAXDIM + 1]
+                )
                 es.enter_context(
                     mock.patch.object(MockPosterior, "rsample", return_value=samples)
                 )
@@ -572,3 +721,21 @@ class TestFidelityUtils(BotorchTestCase):
                 (i + 1) / (num_tr + 1) for i in range(num_tr)
             )
             self.assertTrue(torch.allclose(X.grad, grad_exp))
+
+    def test_project_to_sample_points(self):
+        for batch_shape, dtype in itertools.product(
+            ([], [2]), (torch.float, torch.double)
+        ):
+            q, d, p, d_prime = 1, 12, 7, 4
+            X = torch.rand(*batch_shape, q, d, device=self.device, dtype=dtype)
+            sample_points = torch.rand(p, d_prime, device=self.device, dtype=dtype)
+            X_augmented = project_to_sample_points(X=X, sample_points=sample_points)
+            self.assertEqual(X_augmented.shape, torch.Size(batch_shape + [p, d]))
+            if batch_shape == [2]:
+                self.assertTrue(
+                    torch.allclose(X_augmented[0, :, -d_prime:], sample_points)
+                )
+            else:
+                self.assertTrue(
+                    torch.allclose(X_augmented[:, -d_prime:], sample_points)
+                )
