@@ -39,9 +39,7 @@ class TestSingleTaskGP(BotorchTestCase):
     def _get_model_and_data(
         self, batch_shape, m, outcome_transform=None, input_transform=None, **tkwargs
     ):
-        train_X, train_Y = _get_random_data(
-            batch_shape=batch_shape, num_outputs=m, **tkwargs
-        )
+        train_X, train_Y = _get_random_data(batch_shape=batch_shape, m=m, **tkwargs)
         model_kwargs = {
             "train_X": train_X,
             "train_Y": train_Y,
@@ -51,21 +49,19 @@ class TestSingleTaskGP(BotorchTestCase):
         model = SingleTaskGP(**model_kwargs)
         return model, model_kwargs
 
-    def test_gp(self):
+    def test_gp(self, double_only: bool = False):
         bounds = torch.tensor([[-1.0], [1.0]])
         for batch_shape, m, dtype, use_octf, use_intf in itertools.product(
             (torch.Size(), torch.Size([2])),
             (1, 2),
-            (torch.float, torch.double),
+            (torch.double,) if double_only else (torch.float, torch.double),
             (False, True),
             (False, True),
         ):
             tkwargs = {"device": self.device, "dtype": dtype}
             octf = Standardize(m=m, batch_shape=batch_shape) if use_octf else None
             intf = (
-                Normalize(
-                    d=1, bounds=bounds.to(**tkwargs), transform_on_set_train_data=True
-                )
+                Normalize(d=1, bounds=bounds.to(**tkwargs), transform_on_train=True)
                 if use_intf
                 else None
             )
@@ -74,7 +70,7 @@ class TestSingleTaskGP(BotorchTestCase):
                 m=m,
                 outcome_transform=octf,
                 input_transform=intf,
-                **tkwargs
+                **tkwargs,
             )
             mll = ExactMarginalLogLikelihood(model.likelihood, model).to(**tkwargs)
             with warnings.catch_warnings():
@@ -174,7 +170,7 @@ class TestSingleTaskGP(BotorchTestCase):
             fant_shape = torch.Size([2])
             # fantasize at different input points
             X_fant, Y_fant = _get_random_data(
-                fant_shape + batch_shape, m, n=3, **tkwargs
+                batch_shape=fant_shape + batch_shape, m=m, n=3, **tkwargs
             )
             c_kwargs = (
                 {"noise": torch.full_like(Y_fant, 0.01)}
@@ -277,13 +273,29 @@ class TestSingleTaskGP(BotorchTestCase):
             fm = model.fantasize(X=X_f, sampler=sampler, observation_noise=False)
             self.assertIsInstance(fm, model.__class__)
 
+        # check that input transforms are applied to X.
+        tkwargs = {"device": self.device, "dtype": torch.float}
+        intf = Normalize(d=1, bounds=torch.tensor([[0], [10]], **tkwargs))
+        model, _ = self._get_model_and_data(
+            batch_shape=torch.Size(),
+            m=1,
+            input_transform=intf,
+            **tkwargs,
+        )
+        X_f = torch.rand(4, 1, **tkwargs)
+        fm = model.fantasize(X_f, sampler=SobolQMCNormalSampler(num_samples=3))
+        self.assertTrue(
+            torch.allclose(fm.train_inputs[0][:, -4:], intf(X_f).expand(3, -1, -1))
+        )
+
     def test_subset_model(self):
-        for batch_shape, dtype in itertools.product(
-            (torch.Size(), torch.Size([2])), (torch.float, torch.double)
+        for batch_shape, dtype, use_octf in itertools.product(
+            (torch.Size(), torch.Size([2])), (torch.float, torch.double), (True, False)
         ):
             tkwargs = {"device": self.device, "dtype": dtype}
+            octf = Standardize(m=2, batch_shape=batch_shape) if use_octf else None
             model, model_kwargs = self._get_model_and_data(
-                batch_shape=batch_shape, m=2, **tkwargs
+                batch_shape=batch_shape, m=2, outcome_transform=octf, **tkwargs
             )
             subset_model = model.subset_output([0])
             X = torch.rand(torch.Size(batch_shape + torch.Size([3, 1])), **tkwargs)
@@ -306,7 +318,7 @@ class TestSingleTaskGP(BotorchTestCase):
             model, model_kwargs = self._get_model_and_data(
                 batch_shape=batch_shape, m=2, **tkwargs
             )
-            training_data = TrainingData(
+            training_data = TrainingData.from_block_design(
                 X=model_kwargs["train_X"], Y=model_kwargs["train_Y"]
             )
             data_dict = model.construct_inputs(training_data)
@@ -318,9 +330,7 @@ class TestFixedNoiseGP(TestSingleTaskGP):
     def _get_model_and_data(
         self, batch_shape, m, outcome_transform=None, input_transform=None, **tkwargs
     ):
-        train_X, train_Y = _get_random_data(
-            batch_shape=batch_shape, num_outputs=m, **tkwargs
-        )
+        train_X, train_Y = _get_random_data(batch_shape=batch_shape, m=m, **tkwargs)
         model_kwargs = {
             "train_X": train_X,
             "train_Y": train_Y,
@@ -355,7 +365,7 @@ class TestFixedNoiseGP(TestSingleTaskGP):
             model, model_kwargs = self._get_model_and_data(
                 batch_shape=batch_shape, m=2, **tkwargs
             )
-            training_data = TrainingData(
+            training_data = TrainingData.from_block_design(
                 X=model_kwargs["train_X"],
                 Y=model_kwargs["train_Y"],
                 Yvar=model_kwargs["train_Yvar"],
@@ -368,7 +378,7 @@ class TestFixedNoiseGP(TestSingleTaskGP):
                 torch.equal(data_dict["train_Yvar"], model_kwargs["train_Yvar"])
             )
             # if Yvars is missing, then raise error
-            training_data = TrainingData(
+            training_data = TrainingData.from_block_design(
                 X=model_kwargs["train_X"], Y=model_kwargs["train_Y"]
             )
             with self.assertRaises(ValueError):
@@ -380,9 +390,7 @@ class TestHeteroskedasticSingleTaskGP(TestSingleTaskGP):
         self, batch_shape, m, outcome_transform=None, input_transform=None, **tkwargs
     ):
         with manual_seed(0):
-            train_X, train_Y = _get_random_data(
-                batch_shape=batch_shape, num_outputs=m, **tkwargs
-            )
+            train_X, train_Y = _get_random_data(batch_shape=batch_shape, m=m, **tkwargs)
         train_Yvar = (0.1 + 0.1 * torch.rand_like(train_Y)) ** 2
         model_kwargs = {
             "train_X": train_X,
@@ -394,6 +402,9 @@ class TestHeteroskedasticSingleTaskGP(TestSingleTaskGP):
         model = HeteroskedasticSingleTaskGP(**model_kwargs)
         return model, model_kwargs
 
+    def test_gp(self):
+        super().test_gp(double_only=True)
+
     def test_heteroskedastic_likelihood(self):
         for batch_shape, m, dtype in itertools.product(
             (torch.Size(), torch.Size([2])), (1, 2), (torch.float, torch.double)
@@ -424,6 +435,7 @@ class TestHeteroskedasticSingleTaskGP(TestSingleTaskGP):
 
 
 def _get_pvar_expected(posterior, model, X, m):
+    X = model.transform_inputs(X)
     lh_kwargs = {}
     if isinstance(model.likelihood, FixedNoiseGaussianLikelihood):
         lh_kwargs["noise"] = model.likelihood.noise.mean().expand(X.shape[:-1])
