@@ -11,17 +11,27 @@ import warnings
 import torch
 from botorch.exceptions.warnings import OptimizationWarning
 from botorch.fit import fit_gpytorch_model
-from botorch.models.multitask import FixedNoiseMultiTaskGP, MultiTaskGP
+from botorch.models.multitask import (
+    FixedNoiseMultiTaskGP,
+    KroneckerMultiTaskGP,
+    MultiTaskGP,
+)
 from botorch.models.transforms.input import Normalize
+from botorch.models.transforms.outcome import Standardize
 from botorch.posteriors import GPyTorchPosterior
 from botorch.utils.containers import TrainingData
 from botorch.utils.testing import BotorchTestCase
 from gpytorch.distributions import MultitaskMultivariateNormal, MultivariateNormal
-from gpytorch.kernels import IndexKernel, MaternKernel, ScaleKernel
-from gpytorch.likelihoods import FixedNoiseGaussianLikelihood, GaussianLikelihood
+from gpytorch.kernels import IndexKernel, MaternKernel, MultitaskKernel, ScaleKernel
+from gpytorch.likelihoods import (
+    FixedNoiseGaussianLikelihood,
+    GaussianLikelihood,
+    MultitaskGaussianLikelihood,
+)
 from gpytorch.means import ConstantMean
+from gpytorch.means import MultitaskMean
 from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikelihood
-from gpytorch.priors import GammaPrior
+from gpytorch.priors import GammaPrior, SmoothedBoxPrior
 from gpytorch.priors.lkj_prior import LKJCovariancePrior
 
 
@@ -43,10 +53,16 @@ def _get_model(input_transform=None, **tkwargs):
     return _get_model_and_training_data(input_transform=input_transform, **tkwargs)[0]
 
 
-def _get_model_and_training_data(input_transform=None, **tkwargs):
+def _get_model_and_training_data(
+    input_transform=None, outcome_transform=None, **tkwargs
+):
     train_X, train_Y = _get_random_mt_data(**tkwargs)
     model = MultiTaskGP(
-        train_X, train_Y, task_feature=1, input_transform=input_transform
+        train_X,
+        train_Y,
+        task_feature=1,
+        input_transform=input_transform,
+        outcome_transform=outcome_transform,
     )
     return model.to(**tkwargs), train_X, train_Y
 
@@ -109,22 +125,49 @@ def _get_fixed_noise_and_prior_model(**tkwargs):
     return model.to(**tkwargs)
 
 
+def _get_random_kronecker_mt_data(batch_shape=None, **tkwargs):
+    batch_shape = batch_shape or torch.Size()
+    train_X = (
+        torch.linspace(0, 0.95, 10, **tkwargs).unsqueeze(-1).expand(*batch_shape, 10, 1)
+    )
+    train_X = train_X + 0.05 * torch.rand(*batch_shape, 10, 2, **tkwargs)
+    train_y1 = (
+        torch.sin(train_X[..., 0] * (2 * math.pi))
+        + torch.randn_like(train_X[..., 0]) * 0.2
+    )
+    train_y2 = (
+        torch.cos(train_X[..., 1] * (2 * math.pi))
+        + torch.randn_like(train_X[..., 0]) * 0.2
+    )
+    train_Y = torch.stack([train_y1, train_y2], dim=-1)
+    return train_X, train_Y
+
+
+def _get_kronecker_model_and_training_data(
+    model_kwargs=None, batch_shape=None, **tkwargs
+):
+    model_kwargs = model_kwargs or {}
+    train_X, train_Y = _get_random_kronecker_mt_data(batch_shape=batch_shape, **tkwargs)
+    model = KroneckerMultiTaskGP(train_X, train_Y, **model_kwargs)
+    return model.to(**tkwargs), train_X, train_Y
+
+
 class TestMultiTaskGP(BotorchTestCase):
     def test_MultiTaskGP(self):
         bounds = torch.tensor([[-1.0, 0.0], [1.0, 1.0]])
-        for dtype, use_intf in itertools.product(
-            (torch.float, torch.double), (False, True)
+        for dtype, use_intf, use_octf in itertools.product(
+            (torch.float, torch.double), (False, True), (False, True)
         ):
             tkwargs = {"device": self.device, "dtype": dtype}
+            octf = Standardize(m=1) if use_octf else None
+
             intf = (
-                Normalize(
-                    d=2, bounds=bounds.to(**tkwargs), transform_on_set_train_data=True
-                )
+                Normalize(d=2, bounds=bounds.to(**tkwargs), transform_on_train=True)
                 if use_intf
                 else None
             )
             model, train_X, _ = _get_model_and_training_data(
-                input_transform=intf, **tkwargs
+                input_transform=intf, outcome_transform=octf, **tkwargs
             )
             self.assertIsInstance(model, MultiTaskGP)
             self.assertEqual(model.num_outputs, 2)
@@ -199,10 +242,15 @@ class TestMultiTaskGP(BotorchTestCase):
             with self.assertRaises(RuntimeError):
                 MultiTaskGP(train_X, train_Y, 0, output_tasks=[2])
 
-            # test error if outcome_transform attribute is present
-            model.outcome_transform = None
-            with self.assertRaises(NotImplementedError):
-                model.posterior(test_x)
+            # test outcome transform
+            if use_octf:
+                # ensure un-transformation is applied
+                tmp_tf = model.outcome_transform
+                del model.outcome_transform
+                p_utf = model.posterior(test_x)
+                model.outcome_transform = tmp_tf
+                expected_var = tmp_tf.untransform_posterior(p_utf).variance
+                self.assertTrue(torch.allclose(posterior_f.variance, expected_var))
 
     def test_MultiTaskGP_single_output(self):
         for dtype in (torch.float, torch.double):
@@ -259,9 +307,7 @@ class TestFixedNoiseMultiTaskGP(BotorchTestCase):
         ):
             tkwargs = {"device": self.device, "dtype": dtype}
             intf = (
-                Normalize(
-                    d=2, bounds=bounds.to(**tkwargs), transform_on_set_train_data=True
-                )
+                Normalize(d=2, bounds=bounds.to(**tkwargs), transform_on_train=True)
                 if use_intf
                 else None
             )
@@ -396,7 +442,7 @@ class TestFixedNoiseMultiTaskGP(BotorchTestCase):
         for dtype in (torch.float, torch.double):
             tkwargs = {"device": self.device, "dtype": dtype}
             model, train_X, train_Y = _get_model_and_training_data(**tkwargs)
-            training_data = TrainingData(X=train_X, Y=train_Y)
+            training_data = TrainingData.from_block_design(X=train_X, Y=train_Y)
             # Test that task features are required.
             with self.assertRaisesRegex(ValueError, "`task_features` required"):
                 model.construct_inputs(training_data)
@@ -444,11 +490,13 @@ class TestFixedNoiseMultiTaskGP(BotorchTestCase):
                 train_Y,
                 train_Yvar,
             ) = _get_fixed_noise_model_and_training_data(**tkwargs)
-            td_no_Yvar = TrainingData(X=train_X, Y=train_Y)
+            td_no_Yvar = TrainingData.from_block_design(X=train_X, Y=train_Y)
             # Test that Yvar is required.
             with self.assertRaisesRegex(ValueError, "Yvar required"):
                 model.construct_inputs(td_no_Yvar)
-            training_data = TrainingData(X=train_X, Y=train_Y, Yvar=train_Yvar)
+            training_data = TrainingData.from_block_design(
+                X=train_X, Y=train_Y, Yvar=train_Yvar
+            )
             # Test that task features are required.
             with self.assertRaisesRegex(ValueError, "`task_features` required"):
                 model.construct_inputs(training_data)
@@ -471,3 +519,165 @@ class TestFixedNoiseMultiTaskGP(BotorchTestCase):
             self.assertTrue(torch.equal(data_dict["train_Yvar"], train_Yvar))
             self.assertEqual(data_dict["task_feature"], 0)
             self.assertIsInstance(data_dict["task_covar_prior"], LKJCovariancePrior)
+
+
+class TestKroneckerMultiTaskGP(BotorchTestCase):
+    def test_KroneckerMultiTaskGP_default(self):
+        bounds = torch.tensor([[-1.0, 0.0], [1.0, 1.0]])
+
+        for batch_shape, dtype, use_intf, use_octf in itertools.product(
+            (torch.Size(),),  # torch.Size([3])), TODO: Fix and test batch mode
+            (torch.float, torch.double),
+            (False, True),
+            (False, True),
+        ):
+            tkwargs = {"device": self.device, "dtype": dtype}
+
+            octf = Standardize(m=2) if use_octf else None
+
+            intf = (
+                Normalize(d=2, bounds=bounds.to(**tkwargs), transform_on_train=True)
+                if use_intf
+                else None
+            )
+
+            # initialization with default settings
+            model, train_X, _ = _get_kronecker_model_and_training_data(
+                model_kwargs={"outcome_transform": octf, "input_transform": intf},
+                batch_shape=batch_shape,
+                **tkwargs,
+            )
+            self.assertIsInstance(model, KroneckerMultiTaskGP)
+            self.assertEqual(model.num_outputs, 2)
+            self.assertIsInstance(model.likelihood, MultitaskGaussianLikelihood)
+            self.assertEqual(model.likelihood.rank, 0)
+            self.assertIsInstance(model.mean_module, MultitaskMean)
+            self.assertIsInstance(model.covar_module, MultitaskKernel)
+            base_kernel = model.covar_module
+            self.assertIsInstance(base_kernel.data_covar_module, MaternKernel)
+            self.assertIsInstance(base_kernel.task_covar_module, IndexKernel)
+            task_covar_prior = base_kernel.task_covar_module.IndexKernelPrior
+            self.assertIsInstance(task_covar_prior, LKJCovariancePrior)
+            self.assertEqual(task_covar_prior.correlation_prior.eta, 1.5)
+            self.assertIsInstance(task_covar_prior.sd_prior, SmoothedBoxPrior)
+            lengthscale_prior = base_kernel.data_covar_module.lengthscale_prior
+            self.assertIsInstance(lengthscale_prior, GammaPrior)
+            self.assertEqual(lengthscale_prior.concentration, 3.0)
+            self.assertEqual(lengthscale_prior.rate, 6.0)
+            self.assertEqual(base_kernel.task_covar_module.covar_factor.shape[-1], 2)
+
+            # test model fitting
+            mll = ExactMarginalLogLikelihood(model.likelihood, model)
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore", category=OptimizationWarning)
+                mll = fit_gpytorch_model(mll, options={"maxiter": 1}, max_retries=1)
+
+            # test posterior
+            test_x = torch.rand(2, 2, **tkwargs)
+            posterior_f = model.posterior(test_x)
+            self.assertIsInstance(posterior_f, GPyTorchPosterior)
+            self.assertIsInstance(posterior_f.mvn, MultitaskMultivariateNormal)
+            self.assertEqual(posterior_f.mean.shape, torch.Size([2, 2]))
+            self.assertEqual(posterior_f.variance.shape, torch.Size([2, 2]))
+
+            if use_octf:
+                # ensure un-transformation is applied
+                tmp_tf = model.outcome_transform
+                del model.outcome_transform
+                p_tf = model.posterior(test_x)
+                model.outcome_transform = tmp_tf
+                expected_var = tmp_tf.untransform_posterior(p_tf).variance
+                self.assertTrue(torch.allclose(posterior_f.variance, expected_var))
+            else:
+                # test observation noise
+                # TODO: outcome transform + likelihood noise?
+                posterior_noisy = model.posterior(test_x, observation_noise=True)
+                self.assertTrue(
+                    torch.allclose(
+                        posterior_noisy.variance,
+                        model.likelihood(posterior_f.mvn).variance,
+                    )
+                )
+
+            # test posterior (batch eval)
+            test_x = torch.rand(3, 2, 2, **tkwargs)
+            posterior_f = model.posterior(test_x)
+            self.assertIsInstance(posterior_f, GPyTorchPosterior)
+            self.assertIsInstance(posterior_f.mvn, MultitaskMultivariateNormal)
+            self.assertEqual(posterior_f.mean.shape, torch.Size([3, 2, 2]))
+            self.assertEqual(posterior_f.variance.shape, torch.Size([3, 2, 2]))
+
+    def test_KroneckerMultiTaskGP_custom(self):
+        for batch_shape, dtype in itertools.product(
+            (torch.Size(),),  # torch.Size([3])), TODO: Fix and test batch mode
+            (torch.float, torch.double),
+        ):
+            tkwargs = {"device": self.device, "dtype": dtype}
+
+            # initialization with custom settings
+            likelihood = MultitaskGaussianLikelihood(
+                num_tasks=2,
+                rank=1,
+                batch_shape=batch_shape,
+            )
+            task_covar_prior = LKJCovariancePrior(
+                n=2,
+                eta=0.5,
+                sd_prior=SmoothedBoxPrior(math.exp(-3), math.exp(2), 0.1),
+            )
+            model_kwargs = {
+                "likelihood": likelihood,
+                "task_covar_prior": task_covar_prior,
+                "rank": 1,
+            }
+
+            model, train_X, _ = _get_kronecker_model_and_training_data(
+                model_kwargs=model_kwargs, batch_shape=batch_shape, **tkwargs
+            )
+            self.assertIsInstance(model, KroneckerMultiTaskGP)
+            self.assertEqual(model.num_outputs, 2)
+            self.assertIsInstance(model.likelihood, MultitaskGaussianLikelihood)
+            self.assertEqual(model.likelihood.rank, 1)
+            self.assertIsInstance(model.mean_module, MultitaskMean)
+            self.assertIsInstance(model.covar_module, MultitaskKernel)
+            base_kernel = model.covar_module
+            self.assertIsInstance(base_kernel.data_covar_module, MaternKernel)
+            self.assertIsInstance(base_kernel.task_covar_module, IndexKernel)
+            task_covar_prior = base_kernel.task_covar_module.IndexKernelPrior
+            self.assertIsInstance(task_covar_prior, LKJCovariancePrior)
+            self.assertEqual(task_covar_prior.correlation_prior.eta, 0.5)
+            lengthscale_prior = base_kernel.data_covar_module.lengthscale_prior
+            self.assertIsInstance(lengthscale_prior, GammaPrior)
+            self.assertEqual(lengthscale_prior.concentration, 3.0)
+            self.assertEqual(lengthscale_prior.rate, 6.0)
+            self.assertEqual(base_kernel.task_covar_module.covar_factor.shape[-1], 1)
+
+            # test model fitting
+            mll = ExactMarginalLogLikelihood(model.likelihood, model)
+            with warnings.catch_warnings():
+                warnings.filterwarnings("ignore", category=OptimizationWarning)
+                mll = fit_gpytorch_model(mll, options={"maxiter": 1}, max_retries=1)
+
+            # test posterior
+            test_x = torch.rand(2, 2, **tkwargs)
+            posterior_f = model.posterior(test_x)
+            self.assertIsInstance(posterior_f, GPyTorchPosterior)
+            self.assertIsInstance(posterior_f.mvn, MultitaskMultivariateNormal)
+            self.assertEqual(posterior_f.mean.shape, torch.Size([2, 2]))
+            self.assertEqual(posterior_f.variance.shape, torch.Size([2, 2]))
+
+            # test observation noise
+            posterior_noisy = model.posterior(test_x, observation_noise=True)
+            self.assertTrue(
+                torch.allclose(
+                    posterior_noisy.variance, model.likelihood(posterior_f.mvn).variance
+                )
+            )
+
+            # test posterior (batch eval)
+            test_x = torch.rand(3, 2, 2, **tkwargs)
+            posterior_f = model.posterior(test_x)
+            self.assertIsInstance(posterior_f, GPyTorchPosterior)
+            self.assertIsInstance(posterior_f.mvn, MultitaskMultivariateNormal)
+            self.assertEqual(posterior_f.mean.shape, torch.Size([3, 2, 2]))
+            self.assertEqual(posterior_f.variance.shape, torch.Size([3, 2, 2]))
