@@ -26,10 +26,17 @@ from gpytorch.kernels.scale_kernel import ScaleKernel
 from gpytorch.mlls.exact_marginal_log_likelihood import ExactMarginalLogLikelihood
 from gpytorch.mlls.marginal_log_likelihood import MarginalLogLikelihood
 from gpytorch.mlls.sum_marginal_log_likelihood import SumMarginalLogLikelihood
-from gpytorch.priors.smoothed_box_prior import SmoothedBoxPrior
+from gpytorch.priors.prior import Prior
 from gpytorch.priors.torch_priors import GammaPrior
 
 
+class DummyPrior(Prior):
+    arg_constraints = {}
+
+    def rsample(self, sample_shape=torch.Size()):  # noqa: B008
+        raise NotImplementedError
+
+
 class TestColumnWiseClamp(BotorchTestCase):
     def setUp(self):
         super().setUp()
@@ -65,6 +72,22 @@ class TestColumnWiseClamp(BotorchTestCase):
         X_clmp = columnwise_clamp(X, torch.tensor([-3, -3]), torch.tensor([3, 3]))
         self.assertTrue(torch.equal(X_clmp, X))
 
+    def test_column_wise_clamp_full_dim_tensors(self):
+        X = torch.tensor([[[-1, 2, 0.5], [0.5, 3, 1.5]], [[0.5, 1, 0], [2, -2, 3]]])
+        lower = torch.tensor([[[0, 0.5, 1], [0, 2, 2]], [[0, 2, 0], [1, -1, 0]]])
+        upper = torch.tensor([[[1, 1.5, 1], [1, 4, 3]], [[1, 3, 0.5], [3, 1, 2.5]]])
+        X_expected = torch.tensor(
+            [[[0, 1.5, 1], [0.5, 3, 2]], [[0.5, 2, 0], [2, -1, 2.5]]]
+        )
+        X_clmp = columnwise_clamp(X, lower, upper)
+        self.assertTrue(torch.equal(X_clmp, X_expected))
+        X_clmp = columnwise_clamp(X, lower - 5, upper + 5)
+        self.assertTrue(torch.equal(X_clmp, X))
+        with self.assertRaises(ValueError):
+            X_clmp = columnwise_clamp(X, torch.ones_like(X), torch.zeros_like(X))
+        with self.assertRaises(RuntimeError):
+            X_clmp = columnwise_clamp(X, lower.unsqueeze(-3), upper.unsqueeze(-3))
+
     def test_column_wise_clamp_raise_on_violation(self):
         X = self.X
         with self.assertRaises(BotorchError):
@@ -154,7 +177,7 @@ class TestGetExtraMllArgs(BotorchTestCase):
 class TestExpandBounds(BotorchTestCase):
     def test_expand_bounds(self):
         X = torch.zeros(2, 3)
-        expected_bounds = torch.zeros(1, 3)
+        expected_bounds = torch.zeros(2, 3)
         # bounds is float
         bounds = 0.0
         expanded_bounds = _expand_bounds(bounds=bounds, X=X)
@@ -167,10 +190,21 @@ class TestExpandBounds(BotorchTestCase):
         bounds = torch.zeros(3)
         expanded_bounds = _expand_bounds(bounds=bounds, X=X)
         self.assertTrue(torch.equal(expected_bounds, expanded_bounds))
-        # bounds is > 1-d
+        # bounds is 2-d
         bounds = torch.zeros(1, 3)
         expanded_bounds = _expand_bounds(bounds=bounds, X=X)
         self.assertTrue(torch.equal(expected_bounds, expanded_bounds))
+        # bounds is > 2-d
+        bounds = torch.zeros(1, 1, 3)
+        with self.assertRaises(RuntimeError):
+            # X does not have a t-batch
+            expanded_bounds = _expand_bounds(bounds=bounds, X=X)
+        X = torch.zeros(4, 2, 3)
+        expanded_bounds = _expand_bounds(bounds=bounds, X=X)
+        self.assertTrue(torch.equal(expanded_bounds, torch.zeros_like(X)))
+        with self.assertRaises(RuntimeError):
+            # bounds is not broadcastable to X
+            expanded_bounds = _expand_bounds(bounds=torch.zeros(2, 1, 3), X=X)
         # bounds is None
         expanded_bounds = _expand_bounds(bounds=None, X=X)
         self.assertIsNone(expanded_bounds)
@@ -196,13 +230,13 @@ class TestSampleAllPriors(BotorchTestCase):
             ls = model.covar_module.base_kernel.raw_lengthscale.view(-1).tolist()
             self.assertTrue(all(ls[0] != ls[i]) for i in range(1, len(ls)))
 
-            # change one of the priors to SmoothedBoxPrior
+            # change one of the priors to a dummy prior that does not support sampling
             model.covar_module = ScaleKernel(
                 MaternKernel(
                     nu=2.5,
                     ard_num_dims=model.train_inputs[0].shape[-1],
                     batch_shape=model._aug_batch_shape,
-                    lengthscale_prior=SmoothedBoxPrior(3.0, 6.0),
+                    lengthscale_prior=DummyPrior(),
                 ),
                 batch_shape=model._aug_batch_shape,
                 outputscale_prior=GammaPrior(2.0, 0.15),
@@ -214,7 +248,7 @@ class TestSampleAllPriors(BotorchTestCase):
                 self.assertTrue("rsample" in str(ws[0].message))
 
             # the lengthscale should not have changed because sampling is
-            # not implemented for SmoothedBoxPrior
+            # not implemented for DummyPrior
             self.assertTrue(
                 torch.equal(
                     dict(model.state_dict())[
